// -*-C++-*-

/*
 *  Name         : database.h
 *  Aufgabe      : Implementierung der Datenbank
 *  Version      : $Revision: 1.8 $
 *  Datum        : $Date: 1998/05/13 07:48:38 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef DATABASE_H
#define DATABASE_H

// Forw. Decl
class Found;
class SearchMask;

#include <qobject.h>
#include <qstring.h>

/* Klasse zur Durchfuehrung aller File-Operationen
   auf die Register-Datei einschliesslich der Suche
   nach Eintraegen, die auf das gewuenschte Kriterium
   passen */

class Database : public QObject
{
  Q_OBJECT

public:
  /* Konstruktor: Setzen des Dateinamens der Registerdatei */
  // Registerdatei ist defaultmaessig NICHT nach ISO konvertiert, sondern 
  // im IBM PC Code
  // Wichtig fuer die Umlaute
  Database( QString const & newname );

  /* Durchfuehrung der Suche: File filename oeffnen, Eintrags-
     weise lesen und auf uebereinstimmung mit dem Suchkriterium
     ueberpruefen. Anschliessend File wieder schliessen */
  // gibt anzahl der gefundenen Entries zurueck
  int search(Found *, SearchMask &);

  // Test, ob File filename eine gueltige Registerdatei ist.
  // - gibt es sie ueberhaupt?
  // - kann sie geoeffnet werden?
  // - richtiger Syntax?
  // sollte VOR dem Constructen eines Database Objects aufgerufen werden
  static bool isValid( char const * filename );

  /* Zur bestehenden (oder auch nicht...) Registerdatei
     die neue Datei appendname hinzufuegen. (Inklusive Test,
     ob die Datensaetze der neuen Datei nicht schon in
     der existierenden vorhanden sind */
  /* Aufgerufen von MenuePunkt: Daten anhaengen ... */
  int append(QString & appendName);


public slots:
  /** Slot: Abbruch einer laufenden Suche */
  void abortSearch();


signals:
  /* Signal zur Anzeige der aktuellen Suchposition */
  void databasePosition( int );


private:
  QString filename;

  bool continueSearch;

};

#endif   /* DATABASE_H */

