/*
 *  Name         : dialogs.cpp
 *  Aufgabe      : Widgets: SearchDialog - Dialogfeld zur Festlegung der Parameter einer Suche
 *                          OptionsDialog - Dialogfeld fuer die Voreinstellungen von KHeisePro
 *  Version      : $Revision: 1.39 $
 *  Datum        : $Date: 1999/02/08 22:01:26 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@gmx.de>
 *                     Martin Habbecke   <M.Habbecke@gmx.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <iostream.h>

#include <qstring.h>
#include <qlined.h>
#include <qlabel.h>
#include <qpushbt.h>
#include <qbttngrp.h>
#include <qradiobt.h>
#include <qchkbox.h>
#include <qdialog.h>
#include <qmsgbox.h>
#include <qfontmet.h>
#include <qgrpbox.h>
#include <qaccel.h>
#include <qlayout.h>
#include <qcombo.h>
#include <qstrlist.h>
#include <qdir.h>

#include <kconfig.h>
#include <kapp.h>
#include <kspinbox.h>
#include <kfiledialog.h>
#include <kslider.h>
#include <ktabctl.h>

#include "dialogs.h"
#include "searchmask.h"
#include "database.h"

/**********************************************************
 * Klasse SearchDialog
 **********************************************************/


/* Konstruktor */
SearchDialog::SearchDialog( QWidget *parent = 0, const char *name = 0 )
  : QDialog( parent, name, false )
{
  // Titelzeile setzen
  setCaption( "KHeisePro - Suche" );
  
  /* TopLayout anlegen */
  QVBoxLayout *topLayout = new QVBoxLayout( this, 10, 5 );
  QGridLayout *upperBox = new QGridLayout( 3, 8 );

  topLayout->addLayout( upperBox, 0 );

  upperBox->setColStretch( 1, 1 );
  upperBox->setColStretch( 4, 1 );
  upperBox->setColStretch( 6, 1 );

  /* Label + LineEdit "Schlagwoerter" */
  ESchlagw = new QLineEdit( this );
  ESchlagw->setFixedHeight( ESchlagw->sizeHint().height( ));
  LSchlagw = new QLabel( ESchlagw, "Schlagwrter:", this );
  LSchlagw->setMinimumSize( LSchlagw->sizeHint() );
  BSchlagw = new QPushButton( "&", this );
  BSchlagw->setToggleButton( true );
  BSchlagw->setFixedSize( QSize( ESchlagw->sizeHint().height( ), ESchlagw->sizeHint().height( ) ) );
  connect( BSchlagw, SIGNAL(toggled(bool)), SLOT(toggleSchButton(bool)) );
  // Layouten
  upperBox->addWidget( LSchlagw, 0, 0 );
  upperBox->addMultiCellWidget( ESchlagw, 0, 0, 1, 6 );
  upperBox->addWidget( BSchlagw, 0, 7 );

  /* Label + LineEdit "Autor" */
  EAutor = new QLineEdit( this );
  EAutor->setFixedHeight(EAutor->sizeHint().height() );
  LAutor = new QLabel( EAutor, "Autor:", this );
  LAutor->setMinimumSize( LAutor->sizeHint() );
  BAutor = new QPushButton( "&", this );
  BAutor->setToggleButton( true );
  BAutor->setFixedSize( QSize( EAutor->sizeHint().height( ), EAutor->sizeHint().height( ) ) );
  connect( BAutor, SIGNAL(toggled(bool)), SLOT(toggleAutButton(bool)) );
  // Layouten
  upperBox->addWidget( LAutor, 1, 0 );
  upperBox->addMultiCellWidget( EAutor, 1, 1, 1, 6 );
  upperBox->addWidget( BAutor, 1, 7 );

  /* Label + LineEdit "Redakteur" */
  ERedakteur = new QLineEdit( this );
  ERedakteur->setFixedHeight(ERedakteur->sizeHint().height());
  LRedakteur = new QLabel( ERedakteur, "Redakteur:", this );
  LRedakteur->setMinimumSize( LRedakteur->sizeHint() );
  BRedakteur = new QPushButton( "&", this );
  BRedakteur->setToggleButton( true );
  BRedakteur->setFixedSize( QSize( ERedakteur->sizeHint().height( ), ERedakteur->sizeHint().height( ) ) );  
  connect( BRedakteur, SIGNAL(toggled(bool)), SLOT(toggleRedButton(bool)) );
  // Layouten
  upperBox->addWidget( LRedakteur, 2, 0 );
  upperBox->addWidget( ERedakteur, 2, 1 );
  upperBox->addWidget( BRedakteur, 2, 2 );

  /* Label + LineEdit "Ausgabe" */
  EAusgabe = new QLineEdit( this );
  EAusgabe->setFixedHeight(EAusgabe->sizeHint().height());
  LAusgabe = new QLabel( EAusgabe, "Ausgabe:", this );
  LAusgabe->setMinimumSize( LAusgabe->sizeHint() );
  // Layouten
  upperBox->addWidget( LAusgabe, 2, 3 );
  upperBox->addWidget( EAusgabe, 2, 4 );

    /* Label + LineEdit "Jahrgang" */
  EJahrgang = new QLineEdit( this );
  EJahrgang->setFixedHeight(EJahrgang->sizeHint().height());
  LJahrgang = new QLabel( EJahrgang, "Jahrgang:", this );
  LJahrgang->setMinimumSize( LJahrgang->sizeHint() );
  // Layouten
  upperBox->addWidget( LJahrgang, 2, 5 );
  upperBox->addWidget( EJahrgang, 2, 6 );
 

  /* ButtonGroups layout */
  QHBoxLayout *middleBox = new QHBoxLayout();
  topLayout->addLayout( middleBox, 1 ); 

  /* QButtonGroup fuer Zeitschrift-Typ anlegen */
  GrpZeitschrift = new QButtonGroup( "Zeitschrift", this );
  middleBox->addWidget( GrpZeitschrift, 1 );
  middleBox->addSpacing( 5 );
  // Layout in der Buttongroup erstellen
  QVBoxLayout *box1 = new QVBoxLayout( GrpZeitschrift, 10, 5 );
  box1->addSpacing( GrpZeitschrift->fontMetrics().height() );


  /* Checkbuttons fuer den Zeitschriften-Typ anlegen */
  ZBut1 = new QCheckBox( "c't", GrpZeitschrift );
  ZBut1->setMinimumSize( ZBut1->sizeHint () );
  box1->addWidget( ZBut1, 0 );
  ZBut2 = new QCheckBox( "iX", GrpZeitschrift );
  ZBut2->setMinimumSize( ZBut2->sizeHint () );
  box1->addWidget( ZBut2, 0 );
  ZBut3 = new QCheckBox( "Elrad", GrpZeitschrift );
  ZBut3->setMinimumSize( ZBut3->sizeHint () );
  box1->addWidget( ZBut3, 0 );
  ZBut4 = new QCheckBox( "Gateway", GrpZeitschrift );
  ZBut4->setMinimumSize( ZBut4->sizeHint () );
  box1->addWidget( ZBut4, 0 );
  box1->addStretch( 1 );
  box1->activate();


  /* Hilfs-Layout fuer die rechten beiden Group-Boxen */
  QVBoxLayout *rightBoxes = new QVBoxLayout();
  middleBox->addLayout( rightBoxes, 1 );
  
  /* QButtonGroup fuer Gross/Kleinschreibung anlegen */
  GrpCasitiv = new QButtonGroup( "Gro- / Kleinschreibung", this );
  rightBoxes->addWidget( GrpCasitiv, 1 );
  // Layout fuer Buttonbox erstellen
  QVBoxLayout *box2 = new QVBoxLayout( GrpCasitiv, 10, 5 );
  box2->addSpacing( GrpCasitiv->fontMetrics().height() );


  /* Radiobuttons fuer die Case-Sensitivitaet anlegen */
  CBut1 = new QRadioButton( "beachten", GrpCasitiv );
  CBut1->setMinimumSize( CBut1->sizeHint() );
  box2->addWidget( CBut1, 0 );
  CBut2 = new QRadioButton( "nicht beachten", GrpCasitiv );
  CBut2->setMinimumSize( CBut2->sizeHint() );
  box2->addWidget( CBut2, 0 );
  box2->addStretch( 1 );
  box2->activate();


  /* GroupBox fuer die Fehler-Toleranz anlegen */
  GrpTolerance = new QGroupBox( "Fehler-Toleranz", this );
  rightBoxes->addWidget( GrpTolerance, 1 );
  // Layout fuer GroupBox erstellen
  QVBoxLayout *box3 = new QVBoxLayout( GrpTolerance, 10, 5 );
  box3->addSpacing( GrpTolerance->fontMetrics().height() );

  /* CheckBox, Label und Slider der Toleranz anlegen */
  doTolerance = new QCheckBox( "Fehlertolerant suchen", GrpTolerance );
  doTolerance->setMinimumSize( doTolerance->sizeHint() );
  box3->addWidget( doTolerance, 0 );
  /* Connection: Wenn die Fehler-Toleranz eingeschaltet wird, 
     wird in der ListBox auch die Rating-Spalte angezeigt */
  connect( doTolerance, SIGNAL(clicked()), this, SLOT(refreshTolerance()) );

  labelTolerance = new QLabel( "bereinstimmung:", GrpTolerance );
  labelTolerance->setMinimumSize( labelTolerance->sizeHint() );
  box3->addWidget( labelTolerance, 0 );
  sliderTolerance = new KSlider( 0, 100, 20, 100, KSlider::Horizontal, GrpTolerance );
  sliderTolerance->setTickInterval( 10 );
  sliderTolerance->setMinimumHeight( sliderTolerance->sizeHint().height() );
  box3->addWidget( sliderTolerance, 0 );
  /* Hilfs-Layout fuer zwei horizontale Labels */
  QHBoxLayout *hlp = new QHBoxLayout();
  box3->addLayout( hlp, 1 );
  /* Lalels "0 %" und "100 %" */
  label0Tol = new QLabel( "0 %", GrpTolerance );
  label0Tol->setMinimumSize( label0Tol->sizeHint() );
  hlp->addWidget( label0Tol, 0 );
  hlp->addStretch( 1 );
  label100Tol = new QLabel( "100 %", GrpTolerance );
  label100Tol->setMinimumSize( label100Tol->sizeHint() );
  hlp->addWidget( label100Tol, 0 );
  box3->activate();


  /* QPushButtons anlegen */
  QHBoxLayout *lowerBox = new QHBoxLayout( 10 );
  topLayout->addSpacing( 5 );
  topLayout->addLayout( lowerBox, 0 );


  Schliessen = new QPushButton( "Schlieen", this );
  Schliessen->setFixedHeight( Schliessen->sizeHint().height() );
  connect( Schliessen, SIGNAL(clicked()), this, SLOT(close()) );
  //Layouten
  lowerBox->addWidget( Schliessen, 1 );

  Start = new QPushButton( "Start !", this );
  Start->setFixedHeight( Start->sizeHint().height() );
  connect( Start, SIGNAL(clicked()), this, SIGNAL(STARTclicked()) );
  //Layouten
  lowerBox->addWidget( Start, 1 );

  Default = new QPushButton( "Default", this );
  Default->setFixedHeight( Default->sizeHint().height() );
  connect( Default, SIGNAL(clicked()), this, SIGNAL(DEFAULTclicked()) );
  //Layouten
  lowerBox->addWidget( Default, 1 );

  Loeschen = new QPushButton( "Lschen", this );
  Loeschen->setFixedHeight( Loeschen->sizeHint().height() );
  connect( Loeschen, SIGNAL(clicked()), this, SLOT(clear()) );
  //Layouten
  lowerBox->addWidget( Loeschen, 1 );

  Hilfe = new QPushButton( "Hilfe", this );
  Hilfe->setFixedHeight( Hilfe->sizeHint().height() );
  connect( Hilfe, SIGNAL(clicked()), this, SLOT(help()) );
  //Layouten
  lowerBox->addWidget( Hilfe, 1 );

  /* Breitesten Button herausfinden */
  int butWidth = Start->sizeHint().width();
  if( butWidth < Default->sizeHint().width() )
    butWidth = Default->sizeHint().width();
  if( butWidth < Loeschen->sizeHint().width() )
    butWidth = Loeschen->sizeHint().width();
  if( butWidth < Schliessen->sizeHint().width() )
    butWidth = Schliessen->sizeHint().width();
  if( butWidth < Hilfe->sizeHint().width() )
    butWidth = Hilfe->sizeHint().width();

  /* Alle Buttons auf die gleiche minimale Breite setzen */
  Schliessen->setMinimumWidth( butWidth );
  Start->setMinimumWidth( butWidth );
  Default->setMinimumWidth( butWidth );
  Loeschen->setMinimumWidth( butWidth );
  Hilfe->setMinimumWidth( butWidth );

  /* Start-Button ist zu Anfang disabled */
  Start->setEnabled( false );

  /* Haupt-Layout aktivieren */
  topLayout->activate();

  /* Acceleratoren anlegen */
  Accelerator = new QAccel( this );
  /* Items einfuegen */
  ReturnId = Accelerator->insertItem( Key_Return );
  Accelerator->connectItem( ReturnId, this, SIGNAL(STARTclicked()) );
  Accelerator->setItemEnabled( ReturnId, false );

  /* Groesse des Dialoges festlegen */
  // Minsize in y-Richtung
  resize( 500, 0 );

  /* Dialog verstecken, wird erst gezeigt, wenn
     gesucht werden soll */
  hide();
}


// Moved to "inline"
/* Slot: Togglen der Verknuepfungs-Buttons */
//void SearchDialog::toggleConButton( bool on, QPushButton *but )
//{
//  but->setText( (on) ? ("|") : ("&") );

  //  if( on )
  //    but->setText( "|" );
  //  else
  //    but->setText( "&" );
//}



/* Slot: Reset */
void SearchDialog::receiveSearchOpts( SearchMask searchOpts )
{
  // cout << "SearchDialog::receiveSearchOpts" << endl;
  ESchlagw->setText( searchOpts.Schlagwoerter() );  
  BSchlagw->setOn( !searchOpts.SchlagwoerterMode() );
  EAutor->setText( searchOpts.Autor() );
  BAutor->setOn( !searchOpts.AutorMode() );
  ERedakteur->setText( searchOpts.Redakteur() );
  BRedakteur->setOn( !searchOpts.RedakteurMode() );
  EAusgabe->setText( searchOpts.Ausgabe() );
  EJahrgang->setText( searchOpts.Jahrgang() );

  int Zeitschrift = searchOpts.Zeitschrift();

  ZBut1->setChecked( Zeitschrift & (Entry::CT) );
  ZBut2->setChecked( Zeitschrift & (Entry::IX) );
  ZBut3->setChecked( Zeitschrift & (Entry::EL) );
  ZBut4->setChecked( Zeitschrift & (Entry::GW) );
  
  CBut1->setChecked( searchOpts.isCaseSensitive() );
  CBut2->setChecked( !searchOpts.isCaseSensitive() );

  if( searchOpts.Rating() == -1 )
    {
      doTolerance->setChecked( false );
      //      sliderTolerance->setValue( 100 );
    }
  else
    {
      doTolerance->setChecked( true );
      sliderTolerance->setValue( searchOpts.Rating() );
    }

  refreshTolerance();

  ESchlagw->setFocus();
  // cout << "SearchDialog::receiveSearchOpts ende" << endl;
}



/* Slot: Loeschen */
void SearchDialog::clear( void )
{
  ESchlagw->setText( "" );
  BSchlagw->setOn( false );
  EAutor->setText( "" );
  BAutor->setOn( false );
  ERedakteur->setText( "" );
  BRedakteur->setOn( false );
  EAusgabe->setText( "" );
  EJahrgang->setText( "" );

  ZBut1->setChecked( true );
  ZBut2->setChecked( false );
  ZBut3->setChecked( false );
  ZBut4->setChecked( false );

  CBut1->setChecked( false );
  CBut2->setChecked( true );

  doTolerance->setChecked( false );
  sliderTolerance->setValue( 100 );
  refreshTolerance();
}



/* Fkt zur Speicherung des Inhalts des Dialgs
   in ein SearchMask-Objekt */
void SearchDialog::getContent( SearchMask & dest )
{
  dest.setCaseSensitive( CBut1->isChecked() );

  int Zeitschrift = 0;

  if( ZBut1->isChecked() )
    Zeitschrift |= Entry::CT;
  if( ZBut2->isChecked() )
    Zeitschrift |= Entry::IX;
  if( ZBut3->isChecked() )
    Zeitschrift |= Entry::EL;
  if( ZBut4->isChecked() )
    Zeitschrift |= Entry::GW;

  dest.Schlagwoerter( ESchlagw->text() );
  dest.SchlagwoerterMode( !BSchlagw->isOn() );

  dest.Autor( EAutor->text() );
  dest.AutorMode( !BAutor->isOn() );

  dest.Redakteur( ERedakteur->text() );
  dest.RedakteurMode( !BRedakteur->isOn() );

  dest.Ausgabe( EAusgabe->text() );

  dest.Jahrgang( EJahrgang->text() );

  dest.Zeitschrift( (Entry::MagazinTyp) Zeitschrift );

  dest.Rating( doTolerance->isChecked() ? 
	       ( (sliderTolerance->value()==100) ? (-1) : (sliderTolerance->value()) ) :
	       ( -1 ) );
}


/* Testen, ob die Suchkriterien ausreichen */
bool SearchDialog::checkConsistency( void )
{
  if( !ZBut1->isChecked() && !ZBut2->isChecked() 
      && !ZBut3->isChecked() && !ZBut4->isChecked() )
    {
      QMessageBox::information( this, "KHeisePro", 
				"Es mu mindestens ein Zeitschriften-Typ\n"
				"ausgewhlt werden.\n" );
      return false;
    }
  else if( !strlen( ESchlagw->text() ) && !strlen( EAutor->text() ) 
	   && !strlen( ERedakteur->text() ) )
    {
      QMessageBox::information( this, "KHeisePro",
				"Zur Suche sind Angaben in mindestens einem der Felder\n"
				"\"Schlagwrter\", \"Autor\" oder \"Redakteur\" erforderlich." );
      return false;
    }

    return true;
}


/* Togglen der Toleranz-Widgets */
void SearchDialog::refreshTolerance( void )
{
  bool enab = doTolerance->isChecked();

  labelTolerance->setEnabled( enab );
  label0Tol->setEnabled( enab );
  label100Tol->setEnabled( enab );
  sliderTolerance->setEnabled( enab );

  //  if( !enab )
  //    sliderTolerance->setValue( 100 );

  emit doToleranceClicked( enab );
}







/**********************************************************
 * Klasse OptionsDialog
 **********************************************************/


OptionsDialog::OptionsDialog( QWidget *parent = 0, const char *name = 0 )
  : QDialog( parent, name, false )
{

  // Titel setzen
  setCaption( "KHeisePro - Voreinstellungen" );

  // KTabCtl einbetten
  tabCtl = new KTabCtl( this, "dialogTabCtl" );
  connect( tabCtl, SIGNAL(tabSelected(int)), 
	   this, SLOT(tabChanged(int)) );

  /* TopLayout anlegen */
  QVBoxLayout *topLayout = new QVBoxLayout( this, 10, 5 );
  topLayout->addWidget( tabCtl, 1 );



  /********************************************
   * erster Tab: Such-Optionen
   ********************************************/

  /* GroupBox um die Felder der Such-Optionen */
  searchBox = new QWidget( tabCtl, "Suche" );
  //  topLayout->addWidget( searchBox, 1 );

  QVBoxLayout *innerSearch = new QVBoxLayout( searchBox, 10, 5 );
  QGridLayout *upperBox = new QGridLayout( 3, 8 );
  QHBoxLayout *lowerBox = new QHBoxLayout();
  innerSearch->addSpacing( searchBox->fontMetrics().height() );
  innerSearch->addLayout( upperBox, 0 );
  innerSearch->addLayout( lowerBox, 1 );

  upperBox->setColStretch( 1, 1 );
  upperBox->setColStretch( 4, 1 );
  upperBox->setColStretch( 6, 1 );


  /* Label + LineEdit "Schlagwoerter" */
  ESchlagw = new QLineEdit( searchBox );
  ESchlagw->setFixedHeight(ESchlagw->sizeHint().height());
  LSchlagw = new QLabel( ESchlagw, "Schlagwrter:", searchBox );
  LSchlagw->setMinimumSize( LSchlagw->sizeHint() );
  BSchlagw = new QPushButton( "&", searchBox );
  BSchlagw->setToggleButton( true );
  BSchlagw->setFixedSize( QSize( ESchlagw->sizeHint().height( ), ESchlagw->sizeHint().height( ) ) );
  connect( BSchlagw, SIGNAL(toggled(bool)), SLOT(toggleSchButton(bool)) );
  // Layouten
  upperBox->addWidget( LSchlagw, 0, 0 );
  upperBox->addMultiCellWidget( ESchlagw, 0, 0, 1, 6 );
  upperBox->addWidget( BSchlagw, 0, 7 );

  /* Label + LineEdit "Autor" */
  EAutor = new QLineEdit( searchBox );
  EAutor->setFixedHeight(EAutor->sizeHint().height() );
  LAutor = new QLabel( EAutor, "Autor:", searchBox );
  LAutor->setMinimumSize( LAutor->sizeHint() );
  BAutor = new QPushButton( "&", searchBox );
  BAutor->setToggleButton( true );
  BAutor->setFixedSize( QSize( EAutor->sizeHint().height( ), EAutor->sizeHint().height( ) ) );
  connect( BAutor, SIGNAL(toggled(bool)), SLOT(toggleAutButton(bool)) );
  // Layouten
  upperBox->addWidget( LAutor, 1, 0 );
  upperBox->addMultiCellWidget( EAutor, 1, 1, 1, 6 );
  upperBox->addWidget( BAutor, 1, 7 );

  /* Label + LineEdit "Redakteur" */
  ERedakteur = new QLineEdit( searchBox );
  ERedakteur->setFixedHeight(ERedakteur->sizeHint().height());
  LRedakteur = new QLabel( ERedakteur, "Redakteur:", searchBox );
  LRedakteur->setMinimumSize( LRedakteur->sizeHint() );
  BRedakteur = new QPushButton( "&", searchBox );
  BRedakteur->setToggleButton( true );
  BRedakteur->setFixedSize( QSize( ERedakteur->sizeHint().height( ), ERedakteur->sizeHint().height( ) ) );  
  connect( BRedakteur, SIGNAL(toggled(bool)), SLOT(toggleRedButton(bool)) );
  // Layouten
  upperBox->addWidget( LRedakteur, 2, 0 );
  upperBox->addWidget( ERedakteur, 2, 1 );
  upperBox->addWidget( BRedakteur, 2, 2 );

  /* Label + LineEdit "Ausgabe" */
  EAusgabe = new QLineEdit( searchBox );
  EAusgabe->setFixedHeight(EAusgabe->sizeHint().height());
  LAusgabe = new QLabel( EAusgabe, "Ausgabe:", searchBox );
  LAusgabe->setMinimumSize( LAusgabe->sizeHint() );
  // Layouten
  upperBox->addWidget( LAusgabe, 2, 3 );
  upperBox->addWidget( EAusgabe, 2, 4 );
  
  /* Label + LineEdit "Jahrgang" */
  EJahrgang = new QLineEdit( searchBox );
  EJahrgang->setFixedHeight(EJahrgang->sizeHint().height());
  LJahrgang = new QLabel( EJahrgang, "Jahrgang:", searchBox );
  LJahrgang->setMinimumSize( LJahrgang->sizeHint() );
  // Layouten
  upperBox->addWidget( LJahrgang, 2, 5 );
  upperBox->addWidget( EJahrgang, 2, 6 );



  /* QButtonGroup fuer Zeitschrift-Typ anlegen */
  GrpZeitschrift = new QButtonGroup( "Zeitschrift", searchBox );
  lowerBox->addWidget( GrpZeitschrift, 1 );
  lowerBox->addSpacing( 5 );
  QVBoxLayout *innerLi = new QVBoxLayout( GrpZeitschrift, 10, 5 );
  innerLi->addSpacing( GrpZeitschrift->fontMetrics().height() );

  /* Checkbuttons fuer den Zeitschriften-Typ anlegen */
  ZBut1 = new QCheckBox( "c't", GrpZeitschrift );
  ZBut1->setMinimumSize( ZBut1->sizeHint() );
  innerLi->addWidget( ZBut1, 0 );
  ZBut2 = new QCheckBox( "iX", GrpZeitschrift );
  ZBut2->setMinimumSize( ZBut2->sizeHint() );
  innerLi->addWidget( ZBut2, 0 );
  ZBut3 = new QCheckBox( "Elrad", GrpZeitschrift );
  ZBut3->setMinimumSize( ZBut3->sizeHint() );
  innerLi->addWidget( ZBut3, 0 );
  ZBut4 = new QCheckBox( "Gateway", GrpZeitschrift );
  ZBut4->setMinimumSize( ZBut4->sizeHint() );
  innerLi->addWidget( ZBut4, 0 );
  innerLi->addStretch( 1 );
  innerLi->activate();



  /* Hilfslayout fuer die rechten beiden Boxen anlegen */
  QVBoxLayout *rightBoxes = new QVBoxLayout();
  lowerBox->addLayout( rightBoxes, 1 );
  
  /* QButtonGroup fuer Gross/Kleinschreibung anlegen */
  GrpCasitiv = new QButtonGroup( "Gro- / Kleinschreibung", searchBox );
  rightBoxes->addWidget( GrpCasitiv, 1 );
  QVBoxLayout *innerReUp = new QVBoxLayout( GrpCasitiv, 10, 5 );
  innerReUp->addSpacing( GrpCasitiv->fontMetrics().height() );

  /* Radiobuttons fuer die Case-Sensitivitaet anlegen */
  CBut1 = new QRadioButton( "beachten", GrpCasitiv );
  CBut1->setMinimumSize( CBut1->sizeHint() );
  innerReUp->addWidget( CBut1, 0 );
  CBut2 = new QRadioButton( "nicht beachten", GrpCasitiv );
  CBut2->setMinimumSize( CBut2->sizeHint() );
  innerReUp->addWidget( CBut2, 0 );
  innerReUp->addStretch( 1 );
  innerReUp->activate();


  /* GroupBox fuer die Fehler-Toleranz anlegen */
  GrpTolerance = new QGroupBox( "Fehler-Toleranz", searchBox );
  rightBoxes->addWidget( GrpTolerance, 1 );
  // Layout fuer GroupBox erstellen
  QVBoxLayout *innerReLo = new QVBoxLayout( GrpTolerance, 10, 5 );
  innerReLo->addSpacing( GrpTolerance->fontMetrics().height() );

  /* CheckBox, Label und Slider der Toleranz anlegen */
  doTolerance = new QCheckBox( "Fehlertolerant suchen", GrpTolerance );
  doTolerance->setMinimumSize( doTolerance->sizeHint() );
  innerReLo->addWidget( doTolerance, 0 );
  /* Connection: Wenn die Fehler-Toleranz eingeschaltet wird, 
     wird in der ListBox auch die Rating-Spalte angezeigt */
  connect( doTolerance, SIGNAL(clicked()), this, SLOT(refreshTolerance()) );

  labelTolerance = new QLabel( "bereinstimmung:", GrpTolerance );
  labelTolerance->setMinimumSize( labelTolerance->sizeHint() );
  innerReLo->addWidget( labelTolerance, 0 );
  sliderTolerance = new KSlider( 0, 100, 20, 100, KSlider::Horizontal, GrpTolerance );
  sliderTolerance->setTickInterval( 10 );
  sliderTolerance->setMinimumHeight( sliderTolerance->sizeHint().height() );
  innerReLo->addWidget( sliderTolerance, 0 );
  /* Hilfs-Layout fuer zwei horizontale Labels */
  QHBoxLayout *hlp = new QHBoxLayout();
  innerReLo->addLayout( hlp, 1 );
  /* Lalels "0 %" und "100 %" */
  label0Tol = new QLabel( "0 %", GrpTolerance );
  label0Tol->setMinimumSize( label0Tol->sizeHint() );
  hlp->addWidget( label0Tol, 0 );
  hlp->addStretch( 1 );
  label100Tol = new QLabel( "100 %", GrpTolerance );
  label100Tol->setMinimumSize( label100Tol->sizeHint() );
  hlp->addWidget( label100Tol, 0 );

  innerReLo->activate();

  innerSearch->activate();

  /* Button "beachten" anwaehlen */
  CBut1->setChecked( true );

  // Group-Box als ersten Tab registrieren
  tabCtl->addTab( searchBox, "Suche" );



  /********************************************
   * zweiter Tab: Drucker-Optionen
   ********************************************/

  /* QGroupBox fuer die Drucker-Optionen */
  printerBox = new QWidget( tabCtl, "Drucken" );
  //  topLayout->addWidget( printerBox, 0 );
  QVBoxLayout *prnLayout = new QVBoxLayout( printerBox, 10, 5 );
  prnLayout->addSpacing( printerBox->fontMetrics().height() );
  
  QGridLayout *fontSetupLayout = new QGridLayout( 2, 4 );
  prnLayout->addLayout( fontSetupLayout, 0 );
  fontSetupLayout->setColStretch( 0, 3 );
  fontSetupLayout->setColStretch( 1, 1 );
  fontSetupLayout->setColStretch( 2, 1 );
  fontSetupLayout->setColStretch( 3, 5 );
  
  /* QLabels und SpinBoxes fuer Font-Typ und Groesse anlegen */
  fontSizeSpin = new KNumericSpinBox( printerBox );
  fontSizeSpin->setFixedSize( fontSizeSpin->fontMetrics().width('0') * 6,
				fontSizeSpin->sizeHint().height() );
  fontSizeSpin->setRange( 7, 16 );
  fontSizeLab = new QLabel( fontSizeSpin, "Fontgre:", printerBox );
  fontSizeLab->setMinimumSize( fontSizeLab->sizeHint() );
  fontTypeCom = new QComboBox( false, printerBox );
  fontTypeCom->setFixedSize( fontTypeCom->fontMetrics().width("Helvetica") + 30,
				fontTypeCom->sizeHint().height() );
  fontTypeLab = new QLabel( fontTypeCom, "Font:", printerBox );
  fontTypeLab->setMinimumSize( fontTypeLab->sizeHint() );
  // Layouten
  fontSetupLayout->addWidget( fontTypeLab, 0, 1 );
  fontSetupLayout->addWidget( fontTypeCom, 0, 2 );
  fontSetupLayout->addWidget( fontSizeLab, 1, 1 );
  fontSetupLayout->addWidget( fontSizeSpin, 1, 2 );

  prnLayout->addStretch(1);
  prnLayout->activate();

  // Group-Box als ersten Tab registrieren
  tabCtl->addTab( printerBox, "Drucken" );



  /********************************************
   * dritter Tab: ct-Rom-Optionen
   ********************************************/

  /* QGroupBox fuer die ct-Rom Unterstuetzung */
  ctromBox = new QWidget( tabCtl, "c't-ROM Untersttzung" );
  //topLayout->addWidget( ctromBox, 0 );
  QVBoxLayout *ctromLayout = new QVBoxLayout( ctromBox, 10, 5 );
  ctromLayout->addSpacing( ctromBox->fontMetrics().height() );

  /* Radiobutton um Unterstuetzung ein- / auszuschalten */
  supCtrom = new QCheckBox( "c't-ROMs vorhanden", ctromBox );
  supCtrom->setMinimumSize( supCtrom->sizeHint() );
  /* Connection: Wenn auf die Checkbox gecklickt wird, dann werden die
     zugehoerigen Widgets enabled / disabled */
  connect( supCtrom, SIGNAL(clicked()), this, SLOT(supportCtromClicked()) );
  // Layouten
  ctromLayout->addWidget( supCtrom, 0 );

  /* GroupBox fuer den Mount-Point */
  mountGrp = new QGroupBox( "Mount-Point der c't-ROMs", ctromBox );
  ctromLayout->addWidget( mountGrp );
  QVBoxLayout *mountLayout = new QVBoxLayout( mountGrp, 10, 5 );
  mountLayout->addSpacing( ctromBox->fontMetrics().height() );
  QHBoxLayout *mountInner = new QHBoxLayout();
  mountLayout->addLayout( mountInner, 0 );
  /* LineEdit des MountPoints */
  mountEdit = new QLineEdit( mountGrp );
  mountEdit->setFixedHeight( mountEdit->sizeHint().height() );
  // Layouten
  mountInner->addWidget( mountEdit, 1 );
  /* Button zum Auswaehlen eines Mount-Points */
  mountBut = new QPushButton( "Auswhlen", mountGrp );
  mountBut->setFixedSize( mountBut->sizeHint() );
  connect( mountBut, SIGNAL(clicked()), this, SLOT(selectMountPoint()) );
  // Layouten
  mountInner->addWidget( mountBut );

  /* ButtonGroup fuer den Browser */
  browserGrp = new QButtonGroup( "Artikel-Browser", ctromBox );
  ctromLayout->addWidget( browserGrp ); 
  QVBoxLayout *browserLayout = new QVBoxLayout( browserGrp, 10, 5 );
  browserLayout->addSpacing( ctromBox->fontMetrics().height() );
  QHBoxLayout *browserInner = new QHBoxLayout();
  browserLayout->addLayout( browserInner );
  /* Buttons und Line-Edit der Browser-Wahl */
  kfmBut = new QRadioButton( "kfm", browserGrp );
  kfmBut->setMinimumSize( kfmBut->sizeHint() );
  otherBut = new QRadioButton( "anderer:", browserGrp );
  otherBut->setMinimumSize( otherBut->sizeHint() );
  browserEdit = new QLineEdit( browserGrp );
  browserEdit->setFixedHeight( browserEdit->sizeHint().height() );
  // Layouten
  browserInner->addWidget( kfmBut );
  browserInner->addSpacing( 20 );
  browserInner->addWidget( otherBut );
  browserInner->addWidget( browserEdit, 1 );

  browserLayout->activate();
  mountLayout->activate();
  ctromLayout->addStretch(1);
  ctromLayout->activate();

  // Group-Box als dritten Tab registrieren
  tabCtl->addTab( ctromBox, "c't-Roms" );



  /********************************************
   * vierter Tab: Register-Optionen
   ********************************************/

  /* QWidget fuer Register-Optionen */
  registerBox = new QWidget( tabCtl, "Register" );
  //topLayout->addWidget( registerBox, 0 );
  QVBoxLayout *regLayout = new QVBoxLayout( registerBox, 10, 5 );
  regLayout->addSpacing( registerBox->fontMetrics().height() );

  QHBoxLayout *defaultRegisterLayout = new QHBoxLayout();
  regLayout->addLayout( defaultRegisterLayout, 0 );

  /* QLineEdit fuer Registernamen anlegen */
  ERegister = new QLineEdit( registerBox );
  ERegister->setFixedHeight( ERegister->sizeHint().height() );
  connect( ERegister, SIGNAL(returnPressed()), this, SLOT(selectDefaultRegisterEdit()) );
  LRegister = new QLabel( ERegister, "Default-Register:", registerBox );
  LRegister->setMinimumSize( LRegister->sizeHint() );
  // Layouten
  defaultRegisterLayout->addWidget( LRegister, 0 );
  defaultRegisterLayout->addWidget( ERegister, 1 );

  /* Button zur Auswahl des Default Registers */
  Auswaehlen = new QPushButton( "Auswhlen", registerBox );
  Auswaehlen->setFixedSize( Auswaehlen->sizeHint() );
  connect( Auswaehlen, SIGNAL(clicked()), this, SLOT(selectDefaultRegister()) );
  // Layouten
  defaultRegisterLayout->addWidget( Auswaehlen, 0 );



  /* GroupBox fuer den AutoUpdate anlegen */
  GrpUpdate = new QGroupBox( "Automatischer Register-update", registerBox );
  regLayout->addWidget( GrpUpdate, 0 );
  QVBoxLayout *autoUpdateLayout = new QVBoxLayout( GrpUpdate, 10, 5 );
  autoUpdateLayout->addSpacing( GrpUpdate->fontMetrics().height() );

  /* CheckBox fuer dem automatischen Register-Update */
  autoUpdate = new QCheckBox( "Register automatisch updaten", GrpUpdate );
  autoUpdate->setMinimumSize( autoUpdate->sizeHint() );
  /* Connection: Wenn auf die AutoUpdate CheckBox geklickt
     wird, werden die zugehoerigen controls enabled/disabled */
  connect( autoUpdate, SIGNAL( clicked() ), this, SLOT( autoUpdateClicked() ) );
  // Layouten
  autoUpdateLayout->addWidget( autoUpdate, 0 );

  /* Layoutbox fuer Combo-Boxes */
  QGridLayout *comLayout = new QGridLayout( 2, 3 );
  autoUpdateLayout->addLayout( comLayout, 0 );
  comLayout->setColStretch( 0, 0 );
  comLayout->setColStretch( 1, 0 );
  comLayout->setColStretch( 2, 1 );

  /* Spinbox fuer das Update-Intervall */
  timeSpin = new KNumericSpinBox( GrpUpdate );
  timeSpin->setMinimumSize( QSize( timeSpin->fontMetrics().width('0') * 8,
				   timeSpin->sizeHint().height() ) );
  timeSpin->setRange( 1, 60 );
  comLab1 = new QLabel( timeSpin, "Wann: alle", GrpUpdate );
  comLab1->setMinimumSize( comLab1->sizeHint() );
  comLab3 = new QLabel( timeSpin, "Tage", GrpUpdate );
  comLab3->setMinimumSize( comLab3->sizeHint() );
  // Layouten
  comLayout->addWidget( comLab1, 0, 0 );
  comLayout->addWidget( timeSpin, 0, 1 );
  comLayout->addWidget( comLab3, 0, 2 );


  urlCombo = new QComboBox( true, GrpUpdate );
  urlCombo->setFixedHeight( urlCombo->sizeHint().height() );
  comLab2 = new QLabel( urlCombo, "URL:", GrpUpdate );
  comLab2->setMinimumSize( comLab2->sizeHint() );
  // Layouten
  comLayout->addWidget( comLab2, 1, 0 );
  comLayout->addMultiCellWidget( urlCombo, 1, 1, 1, 2 );


  autoUpdateLayout->activate();

  regLayout->addStretch(1);
  regLayout->activate();

  // Group-Box als vierten Tab registrieren
  tabCtl->addTab( registerBox, "Register" );


  /********************************************
   * fuenfter Tab: Clipboard-Optionen
   ********************************************/

  /* QWidget fuer Clipboard-Optionen */
  clipboardBox = new QWidget( tabCtl, "Clipboard" );

  QVBoxLayout *clipLayout = new QVBoxLayout( clipboardBox, 10, 5 );
  clipLayout->addSpacing( clipboardBox->fontMetrics().height() );  


  /* GroupBox des Copy-Modes */
  GrpCopymode = new QButtonGroup( "Aktuellen Eintrag in Clipboard kopieren", clipboardBox );
  clipLayout->addWidget( GrpCopymode, 0 );
  clipLayout->addSpacing( 5 );
  QVBoxLayout *copymodeLayout = new QVBoxLayout( GrpCopymode, 10, 5 );
  copymodeLayout->addSpacing( GrpCopymode->fontMetrics().height() );

  QHBoxLayout *copyHlp = new QHBoxLayout( 5 );
  copymodeLayout->addLayout( copyHlp, 0 );
  
  CPBut1 = new QRadioButton( "automatisch", GrpCopymode );
  CPBut1->setMinimumSize( CPBut1->sizeHint() );
  copyHlp->addWidget( CPBut1, 0 );
  CPBut2 = new QRadioButton( "manuell", GrpCopymode );
  CPBut2->setMinimumSize( CPBut2->sizeHint() );
  copyHlp->addWidget( CPBut2, 0 );
  copymodeLayout->activate();

  /* Separator-String */
  QHBoxLayout *separatorLayout = new QHBoxLayout();
  clipLayout->addLayout( separatorLayout );
  clipLayout->addSpacing( 5 );

  sepString = new QLineEdit( clipboardBox );
  sepString->setFixedHeight( sepString->sizeHint().height() );
  sepLabel = new QLabel( sepString, "Separator-String:", clipboardBox );
  sepLabel->setMinimumSize( sepLabel->sizeHint() );
  separatorLayout->addWidget( sepLabel );
  separatorLayout->addWidget( sepString, 1 );

  /* GroupBox der Entry-Felder */
  GrpEntries = new QButtonGroup( "Zu bercksichtigende Felder", clipboardBox );
  clipLayout->addWidget( GrpEntries, 0 );
  QHBoxLayout *entriesLayout = new QHBoxLayout( GrpEntries, 10, 5 );
  //Hilfslayout fuer rechte und linke Seite 
  QVBoxLayout *entriesLi = new QVBoxLayout();
  QVBoxLayout *entriesRe = new QVBoxLayout();
  entriesLayout->addLayout( entriesLi );
  entriesLayout->addLayout( entriesRe );

  entriesLi->addSpacing( 10 );
  clipTitel = new QCheckBox( "Titel", GrpEntries );
  clipTitel->setMinimumSize( clipTitel->sizeHint() );
  entriesLi->addWidget( clipTitel );
  clipAutor = new QCheckBox( "Autor", GrpEntries );
  clipAutor->setMinimumSize( clipAutor->sizeHint() );
  entriesLi->addWidget( clipAutor );
  clipSeite = new QCheckBox( "Seite", GrpEntries );
  clipSeite->setMinimumSize( clipSeite->sizeHint() );
  entriesLi->addWidget( clipSeite );
  clipZeitschrift = new QCheckBox( "Zeitschrift", GrpEntries );
  clipZeitschrift->setMinimumSize( clipZeitschrift->sizeHint() );
  entriesLi->addWidget( clipZeitschrift );

  entriesRe->addSpacing( 10 );
  clipUntertitel = new QCheckBox( "Untertitel", GrpEntries );
  clipUntertitel->setMinimumSize( clipUntertitel->sizeHint() );
  entriesRe->addWidget( clipUntertitel );
  clipRedakteur = new QCheckBox( "Redakteur", GrpEntries );
  clipRedakteur->setMinimumSize( clipRedakteur->sizeHint() );
  entriesRe->addWidget( clipRedakteur );
  clipAusgabe = new QCheckBox( "Ausgabe", GrpEntries );
  clipAusgabe->setMinimumSize( clipAusgabe->sizeHint() );
  entriesRe->addWidget( clipAusgabe );
  clipVerweise = new QCheckBox( "Verweise", GrpEntries );
  clipVerweise->setMinimumSize( clipVerweise->sizeHint() );
  entriesRe->addWidget( clipVerweise );

  entriesLayout->activate();



  clipLayout->addStretch( 1 );
  clipLayout->activate();

  // Widget als fuenften Tab registrieren
  tabCtl->addTab( clipboardBox, "Clipboard" );



  /********************************************
   * Buttons des Dialogs
   ********************************************/

    /* QPushButtons anlegen */
  QHBoxLayout *buttonBox = new QHBoxLayout( 10 );
  topLayout->addSpacing( 5 );
  topLayout->addLayout( buttonBox, 0 );

  Schliessen = new QPushButton( "Schlieen", this );
  Schliessen->setFixedHeight( Schliessen->sizeHint().height() );
  connect( Schliessen, SIGNAL(clicked()), this, SLOT(closeDlg()) );
  // Layouten
  buttonBox->addWidget( Schliessen, 1 );

  Speichern = new QPushButton( "Alle sichern", this );
  Speichern->setFixedHeight( Speichern->sizeHint().height() );
  connect( Speichern, SIGNAL(clicked()), this, SLOT(saveOpts()) );
  // Layouten
  buttonBox->addWidget( Speichern, 1 );

  Reload = new QPushButton( "Reload", this );
  Reload->setFixedHeight( Reload->sizeHint().height() );
  connect( Reload, SIGNAL(clicked()), this, SLOT(reloadOpts()) );
  // Layouten
  buttonBox->addWidget( Reload, 1 );

  Loeschen = new QPushButton( "Lschen", this );
  Loeschen->setFixedHeight( Loeschen->sizeHint().height() );
  connect( Loeschen, SIGNAL(clicked()), this, SLOT(clearOpts()) );
  // Layouten
  buttonBox->addWidget( Loeschen, 1 );

  Hilfe = new QPushButton( "Hilfe", this );
  Hilfe->setFixedHeight( Hilfe->sizeHint().height() );
  connect( Hilfe, SIGNAL(clicked()), this, SLOT(help()) );
  // Layouten
  buttonBox->addWidget( Hilfe, 1 );

  /* Breitesten Button herausfinden */
  int butWidth = Schliessen->sizeHint().width();
  if( butWidth < Speichern->sizeHint().width() )
    butWidth = Speichern->sizeHint().width();
  if( butWidth < Reload->sizeHint().width() )
    butWidth = Reload->sizeHint().width();
  if( butWidth < Loeschen->sizeHint().width() )
    butWidth = Loeschen->sizeHint().width();
  if( butWidth < Hilfe->sizeHint().width() )
    butWidth = Hilfe->sizeHint().width();

  /* Alle Buttons auf die gleiche minimale Breite setzen */
  Schliessen->setMinimumWidth( butWidth );
  Speichern->setMinimumWidth( butWidth );
  Reload->setMinimumWidth( butWidth );
  Loeschen->setMinimumWidth( butWidth );
  Hilfe->setMinimumWidth( butWidth );

  /* Haupt-Layout aktivieren */
  topLayout->activate();



  /* Groesse des Dialoges festlegen */
  // Minsize in y-Richtung
  resize( 550, 0 );
  
  /* Dialog verstecken, wird erst gezeigt, wenn
     gesucht werden soll */
  hide();

  /* Erstellen der Liste der Update-Zeitpunkte und der 
     Standard-URLs */
  createUpdateInfo();
  autoUpdateClicked();

  //  clearOpts();
}



/* Erstellen der QStrLists der ComboBoxes */
void OptionsDialog::createUpdateInfo( void )
{
  /* Liste der voreingestellten Update-URLs erstellen */
  urlList = new QStrList( true );
  urlList->append( "ftp://ftp.heise.de/pub/ct/register/register.zip" );
  urlList->append( "ftp://ftp.gwdg.de/pub/magazine/ct/register/register.zip" );
  urlList->append( "ftp://ftp.uni-paderborn.de/ftp/disk4/magazin/ct/register/register.zip" );
  urlCombo->insertStrList( urlList );

  /* Liste der moeglichen Drucker-Fonts anlegen */
  fontTypeList = new QStrList;
  fontTypeList->append( "Times" );
  fontTypeList->append( "Helvetica" );
  fontTypeList->append( "Courier" );
  fontTypeCom->insertStrList( fontTypeList );
}



// Moved to "inline"
/* Slot: Togglen der Verknuepfungs-Buttons */
//void OptionsDialog::toggleConButton( bool on, QPushButton *but )
//{
// if( on )
//    but->setText( "|" );
//  else
//    but->setText( "&" );
//}



/* Leschen aller LineEdits und Radio- / Check-Buttons
   und der benutzerdefinierten URLs */
void OptionsDialog::clearOpts( void )
{

  switch( activeTab )
    {
    case 1:
      ESchlagw->setText( "" );
      EAutor->setText( "" );
      ERedakteur->setText( "" );
      EAusgabe->setText( "" );
      EJahrgang->setText( "" );

      BSchlagw->setOn( false );
      BAutor->setOn( false );
      BRedakteur->setOn( false );
      
      ZBut1->setChecked( true );
      ZBut2->setChecked( false );
      ZBut3->setChecked( false );
      ZBut4->setChecked( false );
      
      CBut1->setChecked( false );
      CBut2->setChecked( true );
      
      doTolerance->setChecked( false );
      sliderTolerance->setValue( 100 );
      refreshTolerance();
      break;

    case 2:
      fontSizeSpin->setValue( 11 );
      fontTypeCom->setCurrentItem( 0 );
      break;

    case 3:
      supCtrom->setChecked( false );
      supportCtromClicked();
      mountEdit->setText( "" );

      kfmBut->setChecked( true );
      otherBut->setChecked( false );
      browserEdit->setText( "netscape -remote 'openFile(%s)' || netscape file:%s &" );
      break;

    case 4:
      ERegister->setText( "" );
      autoUpdate->setChecked( false );
      autoUpdateClicked();
      timeSpin->setValue( 30 );
      urlCombo->clear();
      urlCombo->insertStrList( urlList );

    case 5:
      CPBut1->setChecked( false );
      CPBut2->setChecked( true );

      clipTitel->setChecked( true );
      clipUntertitel->setChecked( false );
      clipAutor->setChecked( false );
      clipRedakteur->setChecked( false );
      clipSeite->setChecked( false );
      clipAusgabe->setChecked( false );
      clipZeitschrift->setChecked( false );
      clipVerweise->setChecked( false );
      
      sepString->setText( "| " );
    }
}




/* Nach Klick auf die AutoUpdate - CheckBox
   zugehoerige Elemente enablen/disablen */
void OptionsDialog::autoUpdateClicked( void )
{
  bool enab = autoUpdate->isChecked();

  comLab1->setEnabled( enab );
  comLab2->setEnabled( enab );
  comLab3->setEnabled( enab );
  timeSpin->setEnabled( enab );
}



/* Aufbereiten und Versenden der Such-Optionen */
void OptionsDialog::sendSearchOpts( void )
{
  SearchMask sendMe;

  sendMe.Schlagwoerter( ESchlagw->text() );
  sendMe.SchlagwoerterMode( !BSchlagw->isOn() ); 
  sendMe.Autor( EAutor->text() );
  sendMe.AutorMode( !BAutor->isOn() ); 
  sendMe.Redakteur( ERedakteur->text() );
  sendMe.RedakteurMode( !BRedakteur->isOn() ); 
  sendMe.Ausgabe( EAusgabe->text() );
  sendMe.Jahrgang( EJahrgang->text() );

  sendMe.setCaseSensitive( CBut1->isChecked() );

  int Zeitschrift = 0;

  if( ZBut1->isChecked() )
    Zeitschrift |= Entry::CT;
  if( ZBut2->isChecked() )
    Zeitschrift |= Entry::IX;
  if( ZBut3->isChecked() )
    Zeitschrift |= Entry::EL;
  if( ZBut4->isChecked() )
    Zeitschrift |= Entry::GW;

  sendMe.Zeitschrift( (Entry::MagazinTyp) Zeitschrift );

  sendMe.Rating( doTolerance->isChecked() ? (sliderTolerance->value()) : ( -1 ) );
  
  emit carrySearchOpts( sendMe );
  // cout << "OptionsDialog:: got SearchOpts request ende" << endl;
}



/* Auswaehlen des Default-Registers */
void OptionsDialog::selectDefaultRegister( void )
{
  /* Dateinamen auswaehlen lassen */
  QString fileName = KFileDialog::getOpenFileName();
  
  if( fileName.isEmpty() )
    return; // Benutzer hat abgebrochen
  
  if( ! Database::isValid( fileName ) ) {
    QMessageBox::warning( this, "KHeisePro", 
			  "Es wurde eine ungltige Registerdatei ausgewhlt!" );
    return;
  }

  // hier ist die Datei auf jeden Fall gueltig
  /* In LineEdit einsetzen, falls eine gueltige Datei ausgewaehlt wurde */
  ERegister->setText( fileName );
  emit newDefaultRegister( fileName );
}


/* Slot fuer die Enter-Taste im LineEdit des
   Default-Registers */
void OptionsDialog::selectDefaultRegisterEdit( void )
{
  QString fileName = ERegister->text();

  if( fileName.isEmpty() )
    return; // Benutzer hat abgebrochen
  
  if( ! Database::isValid( fileName ) ) {
    QMessageBox::warning( this, "KHeisePro", 
			  "Es wurde eine ungltige Registerdatei ausgewhlt!" );
    return;
  }

  // hier ist die Datei auf jeden Fall gueltig
  emit newDefaultRegister( fileName );
}


/* Auswaehlen des Mount-Points der Ctroms */
void OptionsDialog::selectMountPoint( void )
{
  /* Directory auswaehlen lassen */
  QString fileName = KFileDialog::getDirectory( "/" );
  
  if( !fileName.isEmpty() )
    /* In LineEdit einsetzen, falls eine Datei ausgewaehlt wurde */
    mountEdit->setText( fileName );
}



/** Slot zur Behandlung eines Klicks auf die 
    Ctrom-Support-Checkbox */
void OptionsDialog::supportCtromClicked( void )
{
  bool enab = supCtrom->isChecked();

  mountGrp->setEnabled( enab );
  mountEdit->setEnabled( enab );
  mountBut->setEnabled( enab );

  browserGrp->setEnabled( enab );
  kfmBut->setEnabled( enab );
  otherBut->setEnabled( enab );
  browserEdit->setEnabled( enab );

  emit( supportCtrom( enab ) );
}



/* Laden der Voreinstellungen mittels kconfig */
void OptionsDialog::loadOptions( int loadMe )
{
  KConfig *config = kapp->getConfig();

  QString entry, hlp;
  int value;
  bool enab;

  if( loadMe == 2 || loadMe == 0 )
    {
      /* Drucker-Font-Typ laden */
      config->setGroup( "Drucken" );
      value = config->readNumEntry( "Font", 0 );
      if( value < 0 || value > 2 )
	value = 0;
      fontTypeCom->setCurrentItem( value );
      
      value = config->readNumEntry( "Font-Groesse", 11 );
      if( value < 7 || value > 16 )
	value = 11;
      fontSizeSpin->setValue( value );
    }

  if( loadMe == 4 || loadMe == 0 )
    {
      /* Default-Register Laden */
      config->setGroup( "Register" );
      if( hlp = config->readEntry( "Default" ) ) {
	// Pruefen, ob der Benutzer vielleicht per Hand Mist eingegeben hat
	// ODER eine falsche Registerdatei ins LineEdit eingetragen hat !!
	if ( !hlp.isEmpty() ) {
	  if( !Database::isValid( hlp ) ) {
	    QMessageBox::warning( this, "KHeisePro", 
				  "Es wurde ein ungltiges Default-Register\n" 
				  "in der Konfigurations-Datei angegeben !\n\n"
				  "Der Eintrag wird ignoriert und auf \"(kein)\" zurckgesetzt.\n"
				//"(Na, wer hat denn da per Hand an der Datei rumgefummelt?? :-)" 
	      );
	    hlp = ""; // Zuruecksetzen
	  } 
	  else
	  {
	    emit newDefaultRegister( hlp );
	  }
	}
	
	ERegister->setText( hlp );
      }
      
      /* Hinzugefuegte URLs fuer den AutoUpdate laden */
      QStrList *optsList = new QStrList( true );
      urlCombo->clear();
      urlCombo->insertStrList( urlList );
      if( config->readListEntry( "BenutzerURLs", *optsList ) > 0 )
	urlCombo->insertStrList( optsList );
      delete optsList;
      
      /* Ausgewaehlte URL lesen */
      urlCombo->setCurrentItem( config->readNumEntry( "AktuelleURL", 0 ) );
      
      /* Zeitintervall fuer AutoUpdate laden */
      timeSpin->setValue( config->readNumEntry( "UpdateIntervall", 30 ) );
      
      /* Laden, ob AutoUpdate gewuenscht ist */
      autoUpdate->setChecked( config->readBoolEntry( "AutoUpdate", false ) );
      autoUpdateClicked();
    }

  if( loadMe == 1 || loadMe == 0 )
    {
      /* Voreinstellungen fuer die Suche laden */
      config->setGroup( "Suche" );
      if( entry = config->readEntry( "Schlagwoerter" ) )
	ESchlagw->setText( entry );
      BSchlagw->setOn( config->readNumEntry( "Schlagw-Verknuepfung", 0 ) );
      if( entry = config->readEntry( "Autor" ) )
	EAutor->setText( entry );
      BAutor->setOn( config->readNumEntry( "Autor-Verknuepfung", 0 ) );
      if( entry = config->readEntry( "Redakteur" ) )
	ERedakteur->setText( entry );
      BRedakteur->setOn( config->readNumEntry( "Redakteur-Verknuepfung", 0 ) );
      if( entry = config->readEntry( "Ausgabe" ) )
	EAusgabe->setText( entry );
      if( entry = config->readEntry( "Jahrgang" ) )
	EJahrgang->setText( entry );
      
      CBut1->setChecked( enab = config->readBoolEntry( "CaseSensitive", false ) );
      CBut2->setChecked( !enab );
      
      entry = config->readEntry( "Zeitschrift", "CT" );
      ZBut1->setChecked( entry.contains( "CT" ) );
      ZBut2->setChecked( entry.contains( "IX" ) );
      ZBut3->setChecked( entry.contains( "EL" ) );
      ZBut4->setChecked( entry.contains( "GW" ) );
	      
      /* Voreinstellungen fuer die Fehlertolerante Suche laden */
      doTolerance->setChecked( config->readBoolEntry( "Fehlertoleranz", false ) );
      sliderTolerance->setValue( config->readNumEntry( "Uebereinstimmung", 100 ) );
      refreshTolerance();
    }

  if( loadMe == 3 || loadMe == 0 )
    {
      /* Voreinstellungen fuer den Ctrom-Support laden */
      config->setGroup( "CTROM" );
      supCtrom->setChecked( config->readBoolEntry( "Support", false ) );
      supportCtromClicked();
      if( entry = config->readEntry( "MountPoint" ) )
	{
	  if( !QDir().cd( entry ) )
	    {
	      QMessageBox::information( this, "KHeisePro",
					"Es wurde ein ungltiger Mount-Point der c't-Roms\n" 
					"in der Konfigurations-Datei angegeben !\n\n"
					"Der Eintrag wird ignoriert und zurckgesetzt.\n" );
	      entry = "";
	    }
	  
	  mountEdit->setText( entry );
	}

      entry = config->readEntry( "Browser", "kfm" );
      kfmBut->setChecked( entry == "kfm" );
      otherBut->setChecked( entry == "anderer" );

      browserEdit->setText( config->readEntry( "Browser Commandline", 
					       "netscape -remote 'openFile(%s)' || netscape file:%s &" ) );
    }

  if( loadMe == 5 || loadMe == 0 )
    {
      /* Voreinstellungen fuer das Clipboard laden */
      config->setGroup( "Clipboard" );
      hlp = config->readEntry( "KopierModus" );
      CPBut1->setChecked( hlp == "automatisch" );
      CPBut2->setChecked( hlp != "automatisch" );
      
      hlp = config->readEntry( "SeparatorString" );
      if( hlp.length() > 2 )
	sepString->setText( hlp.mid( 1, hlp.length() - 2 ) );
      else
	sepString->setText( "|" );
			    
      entry = config->readEntry( "Felder", "TI" );
      clipTitel->setChecked( entry.contains( "TI" ) );
      clipUntertitel->setChecked( entry.contains( "UT" ) );
      clipAutor->setChecked( entry.contains( "AT" ) );
      clipRedakteur->setChecked( entry.contains( "RE" ) );
      clipSeite->setChecked( entry.contains( "SE" ) );
      clipAusgabe->setChecked( entry.contains( "AG" ) );
      clipZeitschrift->setChecked( entry.contains( "ZT" ) );
      clipVerweise->setChecked( entry.contains( "VW" ) );
    }
}



/* Speichern der Voreinstellungen mittels kconfig */
void OptionsDialog::saveOptions( void )
{
  // cout << "saving Options" << endl;

  /* KConfig holen */
  KConfig *config = kapp->getConfig();

  QString entry;

  /* Clipboard-Optionen sichern */
  config->setGroup( "Clipboard" );
  config->writeEntry( "KopierModus", ( CPBut1->isChecked() ? "automatisch" : "manuell" ) );
  entry = "\"";
  entry += sepString->text();
  entry += "\"";
  config->writeEntry( "SeparatorString", entry );
  /* zu beruecksichtigenden Felder aufbereiten */
  entry = "";
  if( clipTitel->isChecked() )
    entry += "TI ";
  if( clipUntertitel->isChecked() )
    entry += "UT ";
  if( clipAutor->isChecked() )
    entry += "AT ";
  if( clipRedakteur->isChecked() )
    entry += "RE ";
  if( clipSeite->isChecked() )
    entry += "SE ";
  if( clipAusgabe->isChecked() )
    entry += "AG ";
  if( clipZeitschrift->isChecked() )
    entry += "ZT ";
  if( clipVerweise->isChecked() )
    entry += "VW ";
  config->writeEntry( "Felder", entry );

  /* Drucker-Optionen sichern */
  config->setGroup( "Drucken" );
  config->writeEntry( "Font", fontTypeCom->currentItem() );
  config->writeEntry( "Font-Groesse", fontSizeSpin->getValue() );

  /* Register-Options speichern */
  config->setGroup( "Register" );
  entry = ERegister->text();
  config->writeEntry( "Default", entry );
  
  /* Benutzerdefinierte URLs fuer den AutoUpdate speichern */
  QStrList *optsList = new QStrList( true );
  for( int i = 0; i < urlCombo->count(); i++ )
    if( urlList->find( urlCombo->text( i ) ) == -1 )
      optsList->append( urlCombo->text( i ) );
  config->writeEntry( "BenutzerURLs", *optsList );
  delete optsList;

  /* Ausgewaehlte URL speichern */
  config->writeEntry( "AktuelleURL", urlCombo->currentItem() );

  /* Eingestelltes UpdateIntervall spechern */
  config->writeEntry( "UpdateIntervall", timeSpin->getValue() );

  /* Speichern, ob AutoUpdate erwuenscht */
  config->writeEntry( "AutoUpdate", autoUpdate->isChecked() );

  /* SearchOpts speichern */
  config->setGroup( "Suche" );
  config->writeEntry( "Schlagwoerter", ESchlagw->text() );
  config->writeEntry( "Schlagw-Verknuepfung", BSchlagw->isOn() ? 1 : 0 );
  
  config->writeEntry( "Autor", EAutor->text() );
  config->writeEntry( "Autor-Verknuepfung", BAutor->isOn() ? 1 : 0 );

  config->writeEntry( "Redakteur", ERedakteur->text() );
  config->writeEntry( "Redakteur-Verknuepfung", BRedakteur->isOn() ? 1 : 0 );
  
  config->writeEntry( "Ausgabe", EAusgabe->text() );

  config->writeEntry( "Jahrgang", EJahrgang->text() );

  /* Case-Sensitivitaet speichern */
  config->writeEntry( "CaseSensitive", CBut1->isChecked() );

  /* Zeitschriften-Typ aufbereiten und abspeichern */
  entry = "";
  if( ZBut1->isChecked() )
    entry.append( "CT " );
  if( ZBut2->isChecked() )
    entry.append( "IX " );
  if( ZBut3->isChecked() )
    entry.append( "EL " );
  if( ZBut4->isChecked() )
    entry.append( "GW" );
  config->writeEntry( "Zeitschrift", entry );

  /* Fehlertoleranz-Opts. speichern */
  config->writeEntry( "Fehlertoleranz", doTolerance->isChecked() );
  config->writeEntry( "Uebereinstimmung", sliderTolerance->value() );
  
  /* Voreinstellungen fuer die Ctrom-Unterstuetzung speichern */
  config->setGroup( "CTROM" );
  config->writeEntry( "Support", supCtrom->isChecked() );
  config->writeEntry( "MountPoint", mountEdit->text() );
  config->writeEntry( "Browser", (kfmBut->isChecked()) ? "kfm" : "anderer" );
  config->writeEntry( "Browser Commandline", browserEdit->text() );
  config->sync();
}


/* Togglen der Toleranz-Widgets */
void OptionsDialog::refreshTolerance( void )
{
  bool enab = doTolerance->isChecked();

  labelTolerance->setEnabled( enab );
  label0Tol->setEnabled( enab );
  label100Tol->setEnabled( enab );
  sliderTolerance->setEnabled( enab );

  //  if( !enab )
  //    sliderTolerance->setValue( 100 );
}


#include "dialogs.moc"
