// Mode: -*-C++-*-

/*
 *  Name         : dialogs.h
 *  Aufgabe      : Widgets: SearchDialog - Dialogfeld zur Festlegung der Parameter einer Suche
 *                          OptionsDialog - Dialogfeld fuer die Voreinstellungen von KHeisePro
 *  Version      : $Revision: 1.25 $
 *  Datum        : $Date: 1999/02/08 22:01:27 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef DIALOGS_H
#define DIALOGS_H


#include <qdialog.h>
#include <qcombo.h>
#include <qpushbt.h>
#include <qchkbox.h>
#include <qradiobutton.h>
#include <qaccel.h>

#include <kspinbox.h>

/* Forward Deklarationen */
class QLabel;
class QLineEdit;
class QButtonGroup;
class QRadioButton;
class QCheckBox;
class QGroupBox;
class SearchMask;
class QComboBox;
class QString;
class QStrList;
class SearchMask;
class KSlider;
class KTabCtl;

/** Klasse des Such-Dialogfeldes. Hier werden die 
 Optionen und Suchstrings fuer eine Suche eingetragen */
class SearchDialog : public QDialog
{
  Q_OBJECT

public:
  /** Konstruktor: SearchDialog erzeugen */
  SearchDialog( QWidget *parent = 0, const char *name = 0 );

  /** Funktion, die den Inhalt des Dialogs in ein Searchmask-
      Objekt speichert */
  void getContent( SearchMask & );

  /** Funktion zum Ueberpruefen der Konsistenz der
      Suchkriterien (wurde mind. ein Magazintyp angewaehlt) */
  bool checkConsistency( void );

public slots:
  /** Slots der Verknuepfungs-Buttons */
  inline void toggleSchButton( bool t ) { toggleConButton( t, BSchlagw ); }
  inline void toggleAutButton( bool t ) { toggleConButton( t, BAutor ); }
  inline void toggleRedButton( bool t ) { toggleConButton( t, BRedakteur ); }
  //  inline void toggleAusButton( bool t ) { toggleConButton( t, BAusgabe ); }
  //  inline void toggleJahButton( bool t ) { toggleConButton( t, BJahrgang ); }

  /** Slot zum togglen der Verknuepfungs-Buttons */
  inline void toggleConButton( bool on, QPushButton *but ) { but->setText( (on) ? ("|") : ("&") ); }

  /** Slot zum Loeschen der Dialogeintraege */
  inline void clear();

  /** Slot zum Schliessen des Suchdialogs */
  void close() { hide(); }

  /** Slot zur Anzeige der Hilfe */
  inline void help() { kapp->invokeHTMLHelp( "kheisepro/index-5.html" , "suchen" ); }

  /** Slot zum Empfangen und Verarbeiten der Voreinstellungen
      aus dem OptionsDialog */
  void receiveSearchOpts( SearchMask );

  /** Slot zum disablen / enblen des "Start"-Buttons */
  void enableStartBut( bool enab ) { Start->setEnabled( enab );
                                     Accelerator->setItemEnabled( ReturnId, enab ); }

  /** Slot zum togglen der Toleranz-Widgets */
  void refreshTolerance();


signals:
  /** Signal, das gesendet wird bei Klick auf Start-Button */
  void STARTclicked();

  /** Signal, das meldet, wenn "Reset" geklickt wurde */
  void DEFAULTclicked();

  /** Signal, wenn auf die CheckBox "Fehlertolerant Suchen"
      gecklickt wurde */
  void doToleranceClicked( bool );


private:
  /** Labels der QLineEdit-Widgets */
  QLabel *LSchlagw, *LAutor, *LRedakteur, *LAusgabe, *LJahrgang;

  /** QPushButtons fuer Verknuepfungsart der Ihalte der
      LineEdits */
  QPushButton *BSchlagw, *BAutor, *BRedakteur;

  /** QLineEdit-Widgets fuer die Suchwoerter */
  QLineEdit *ESchlagw, *EAutor, *ERedakteur, *EAusgabe, *EJahrgang;

  /** Box der Radiobuttons "Zeitschrift" */
  QButtonGroup *GrpZeitschrift;
  QButtonGroup *GrpCasitiv;

  /** GroupBox fuer die Fehler-Toleranz */
  QGroupBox *GrpTolerance;

  /** Radiobuttons fuer die Buttongroups */
  QCheckBox *ZBut1, *ZBut2, *ZBut3, *ZBut4;
  QRadioButton *CBut1, *CBut2;

  /** Labels, Buttons und Slider fuer die Fehler-Toleranz */
  QCheckBox *doTolerance;
  QLabel *labelTolerance;
  KSlider *sliderTolerance;
  QLabel *label0Tol, *label100Tol;

  /** Buttons "Ok", "Zuruecksetzen", "Abbruch", "Hilfe" */
  QPushButton *Schliessen, *Start, *Default, *Loeschen, *Hilfe;

  /** Acceleratoren ( <Enter> = Suche beginnen ) */
  QAccel *Accelerator;
  int ReturnId;
};






/** Klasse fuer das Voreinstellungen-Dialogfeld.
  Hier werden Einstellungen fuer das generelle Verhalten
  des Programms gemacht */
class OptionsDialog : public QDialog
{
  Q_OBJECT


public:
  /** Konstruktor: OptionsDialog erzeugen und Voreinstellungen
      mittels kconfig einlesen */
  OptionsDialog( QWidget *parent = 0, const char *name = 0 );

  /** Funktion zum Ueberpruefen der Konsistenz der
      Suchkriterien (wurde mind. ein Magazintyp angewaehlt) */
  bool checkConsistency( void );

  /** Funktion zur Rueckgabe des Default Registers */
  inline const char * getDefaultRegister( void ) const { return ERegister->text(); }

  /** Funktion zur Rueckgabe der ausgewaehlten URL fuer den
      AutoUpdate */
  inline const char * getUpdateURL( void ) { return urlCombo->currentText(); }

  /** Funktion, die zurueckliefert, ob ein automatischer
      Update gewuenscht wird */
  inline bool getAutoUpdate( void ) { return autoUpdate->isChecked(); }

  /** Funktion zur Rueckgabe des Update-Intervalls fuer
      den AutoUpdate */
  inline int getUpdateIntervall( void ) { return timeSpin->getValue(); }

  /** Funktion zur Reuckgabe des Drucker-Fonts */
  inline const char * getPrinterFont( void ) { return fontTypeCom->currentText(); }

  /** Funktion zur Rueckgabe der Drucker-Font-Groesse */
  inline int getPrinterFontSize( void ) { return fontSizeSpin->getValue(); }

  /** Funktion zur Rueckgabe des Ctrom-MountPoints */
  inline const char * getMountPoint( void ) { return mountEdit->text(); }
  /** Functon, die die gewuenschte Browser-Kommandozeile zurueckgibt */
  inline const char *getBrowser( void ) { return kfmBut->isChecked() ? "kfm" : 
    browserEdit->text(); }

  /** Funktionen, die zurueckgeben, ob ein bestimmtes Feld in das 
      Clipboard kopiert werden soll oder nicht */
  inline bool titelToClipboard( void ) { return clipTitel->isChecked(); }
  inline bool untertitelToClipboard( void ) { return clipUntertitel->isChecked(); }
  inline bool autorToClipboard( void ) { return clipAutor->isChecked(); }
  inline bool redakteurToClipboard( void ) { return clipRedakteur->isChecked(); }
  inline bool seiteToClipboard( void ) { return clipSeite->isChecked(); }
  inline bool ausgabeToClipboard( void ) { return clipAusgabe->isChecked(); }
  inline bool zeitschriftToClipboard( void ) { return clipZeitschrift->isChecked(); }
  inline bool verweiseToClipboard( void ) { return clipVerweise->isChecked(); }
  /** Separator-String fuers Clipboard */
  inline const char *separatorString( void ) { return sepString->text(); }
  /** Daten Automatisch ins Clipboard kopieren ? */
  inline bool autoCopyToClipboard( void ) { return CPBut1->isChecked(); }
  

public slots:
  /** Slots der Verknuepfungs-Buttons */
  inline void toggleSchButton( bool t ) { toggleConButton( t, BSchlagw ); }
  inline void toggleAutButton( bool t ) { toggleConButton( t, BAutor ); }
  inline void toggleRedButton( bool t ) { toggleConButton( t, BRedakteur ); }
  //  inline void toggleAusButton( bool t ) { toggleConButton( t, BAusgabe ); }
  //  inline void toggleJahButton( bool t ) { toggleConButton( t, BJahrgang ); }

  /** Slot zum togglen der Verknuepfungs-Buttons */
  inline void toggleConButton( bool on, QPushButton *but ) { but->setText( (on) ? ("|") : ("&") ); }

  /** Slot zum Beenden des Dialogs */
  inline void closeDlg() { hide(); }

  /** Slot zum Speichern der Voreinstellungen */
  inline void saveOpts() { saveOptions(); }

  /** Slot zum ereuten Laden der Voreinstellungen */
  inline void reloadOpts() { loadOptions( activeTab ); } 

  /** Slot zum Loeschen aller Eintraege */
  void clearOpts();

  /** Slot zur Anzeige der Hilfe */
  inline void help() { kapp->invokeHTMLHelp( "kheisepro/index-5.html", "voreinstellungen" ); }

  /** Slot zum Aufbereiten und versenden der
      Such-Optionen */
  void sendSearchOpts();

  /** Slot zum aussuchen des Default-Registers */
  void selectDefaultRegister();
  
  /** Slot, wenn im LineEdit des Default-Registers
      <Enter> gedrueckt wird */
  void selectDefaultRegisterEdit();

  /** Slot zur Behandlung eines Klicks auf die
      Auto-Update-CheckBox */
  void autoUpdateClicked();

  /** Slot zum Auswaehlen des Mount-Points der Ct-Roms */
  void selectMountPoint();

  /** Slot zur Behandlung eines Klicks auf die 
      Ctrom-Support-Checkbox */
  void supportCtromClicked();

  /** Slot zur Behandlung eines Klicks auf die 
      "Fehlertolerant Suchen"-CheckBox */
  void refreshTolerance();

  /** Laden der Voreinstellungen mittels kconfig */
  void loadOptions( int );

  /** Speichern der Voreinstellungen mittels kconfig */
  void saveOptions( void );

  /** Merken des aktuellen Tabs */
  inline void tabChanged( int tab) { activeTab = tab + 1; }


signals:
  /** Signal zum Transportieren der Such-Optionen
      zum SearchDialog */
  void carrySearchOpts( SearchMask );

  /** Signal zum versenden, ob die Ctroms unterstuetzt
      werden oder nicht */
  void supportCtrom( bool );

  /** Signal zur Anzeige, dass ein neues Default-Register
      gesetzt wurde */
  void newDefaultRegister( const char * );


private:
  /** Erstellen der StrLists fuer die Combo-Boxen */
  void createUpdateInfo( void );


  /** KTabCtl fuer den Dialog */
  KTabCtl *tabCtl;

  /** GroupBox um die Setup-Elemente der Suche */
  QWidget *searchBox;

  /** Labels der QLineEdit-Widgets */
  QLabel *LSchlagw, *LAutor, *LRedakteur, *LAusgabe, *LJahrgang;

  /** QPushButtons fuer Verknuepfungsart der Ihalte der
      LineEdits */
  QPushButton *BSchlagw, *BAutor, *BRedakteur;

  /** QLineEdit-Widgets fuer die Suchwoerter */
  QLineEdit *ESchlagw, *EAutor, *ERedakteur, *EAusgabe, *EJahrgang;

  /** Box der Radiobuttons "Zeitschrift" */
  QButtonGroup *GrpZeitschrift;
  QButtonGroup *GrpCasitiv;

  /** GroupBox fuer die Fehler-Toleranz */
  QGroupBox *GrpTolerance;

  /** Radiobuttons fuer die Buttongroups */
  QCheckBox *ZBut1, *ZBut2, *ZBut3, *ZBut4;
  QRadioButton *CBut1, *CBut2;

  /** Labels, Buttons und Slider fuer die Fehler-Toleranz */
  QCheckBox *doTolerance;
  QLabel *labelTolerance;
  KSlider *sliderTolerance;
  QLabel *label0Tol, *label100Tol;



  /** GroupBox fuer die Drucker-Optionen */
  QWidget *printerBox;

  /** Labels fuer Fontsauswahl und Schriftgreossen auswahl */
  QLabel *fontSizeLab, *fontTypeLab;

  /** KNumericalSpinBox fuer die Fontgroesse */
  KNumericSpinBox *fontSizeSpin;

  /** KSpinBox fuer die Font-Typen */
  QComboBox *fontTypeCom;

  /** QStrList der moeglichen Fonts */
  QStrList *fontTypeList;



  /** GroupBox fuer die ct-Rom-Unterstuetzung */
  QWidget *ctromBox;

  /** Checkbox zum Ein-/Ausschalten */
  QCheckBox *supCtrom;

  /** GroupBox fuer den Mount-Point */
  QGroupBox *mountGrp;
  
  /** LineEdit fuer den Mount-Point */
  QLineEdit *mountEdit;

  /** QPushButton zum oeffnen eines File-Dialogs zur Auswahl
      des Mount-Points */
  QPushButton *mountBut;

  /** GroupBox fuer den gewuenschten Browser */
  QButtonGroup *browserGrp;
  
  /** Zwei RadioButtons */
  QRadioButton *kfmBut, *otherBut;
  
  /** LineEdit zur Eingabe des Browsers */
  QLineEdit *browserEdit;



  /** GroupBox fuer die Register-Optionen */
  QWidget *registerBox;

  /** LineEdit fuer den Namen der Standart-Registerdatei */
  QLineEdit *ERegister;

  /** Laber fuer ERegister */
  QLabel *LRegister;

  /** Button zum Auswaehlen der Default-Register-Datei */
  QPushButton *Auswaehlen;


  /** GroupBox fuer AutoUpdate */
  QGroupBox *GrpUpdate;

  /** CheckBox fuer Auto-Update der Registerdatei */
  QCheckBox *autoUpdate;

  /** Labels der ComboBoxes */
  QLabel *comLab1, *comLab2, *comLab3;

  /** KNumericSpinBox fuer die Einstellung des Update
      Intervalls */
  KNumericSpinBox *timeSpin;

  /** QComboBox fuer die Einstellung der Update-URL */
  QComboBox *urlCombo;  

  /** Liste der Voreingestellten URLs */
  QStrList *urlList;



  /** "Seite" fuer ClipBoard-Optionen */
  QWidget *clipboardBox;

  /** GroupBox fuer Kopier-Modus */
  QButtonGroup *GrpCopymode;

  /** RadioButtons fuer automatisch / manuell */
  QRadioButton *CPBut1, *CPBut2;

  /** Label des Separator-Strings */
  QLabel *sepLabel;

  /** Entry fuer den Separator-String */
  QLineEdit *sepString;

  /** GroupBox der zu beruecksichtigenden Felder */
  QButtonGroup *GrpEntries;

  /** CheckBoxen der Felder von Entry */
  QCheckBox *clipTitel, *clipUntertitel, *clipAutor, *clipRedakteur;
  QCheckBox *clipSeite, *clipAusgabe, *clipZeitschrift, *clipVerweise;



  /** Buttons "Ok", "Reload", "Loechen", "Abbruch", "Hilfe" */
  QPushButton *Schliessen, *Speichern, *Reload, *Loeschen, *Hilfe;

  
  /** Merker fuer den aktuellen Tab */
  int activeTab;
};


#endif // DIALOGS_H
