/*
 *  Name         : entry.cpp
 *  Aufgabe      : Speicherung eines Eintrages der Registerdatei
 *  Version      : $Revision: 1.21 $
 *  Datum        : $Date: 1998/08/24 13:01:12 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


// strtok
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include <stdio.h>

#include <qstrlist.h>

#include "entry.h"
#include "searchmask.h"


// Fuern einfacheren Test auf "leer"
#define isEmpty(c) (c[0] == '\0')

// Konstruktor:
// Speicher fuer maximale Groesse allozieren
// und als leer markieren
Entry::Entry()
{
  initVars();
}

void Entry::initVars( void )
{
  Titel_ = new char[TitelMAX];
  Titel_[0]         = '\0';

  Untertitel_ = new char[UntertitelMAX];
  Untertitel_[0]    = '\0';

  Autor_ = new char[AutorMAX];
  Autor_[0]         = '\0';

  Redakteur_ = new char[RedakteurMAX];
  Redakteur_[0]     = '\0';

  Seite_ = new char[SeiteMAX];
  Seite_[0]         = '\0';

  Ausgabe_ = new char[AusgabeMAX];
  Ausgabe_[0]       = '\0';

  Zeitschrift_      =   CT;

  Jahrgang_ = new char[JahrgangMAX];
  Jahrgang_[0]      = '\0';

  Querverweise_ = new char[QuerverweiseMAX];
  Querverweise_[0]  = '\0';

  Schlagwoerter_ = new char[SchlagwoerterMAX];
  Schlagwoerter_[0] = '\0';

  // Default Rating -> -1
  // Heisst: Kein Rating vorhanden
  // Fuer das Sortieren einfacher
  // d.h. wenn keine Textsensitive Suche durchgefuehrt wird, dann sind alle gleich!
  Rating( -1 );
}

// Copy-Konstruktor
// es wird gerade soviel kopiert, wie noetig ist
Entry::Entry( Entry const & a)
{
  Titel_ = new char[ strlen( a.Titel_ ) + 1 ];
  strcpy( Titel_,  a.Titel_ );

  Untertitel_ = new char[ strlen(  a.Untertitel_ ) + 1 ];
  strcpy( Untertitel_,  a.Untertitel_ );

  Autor_ = new char[ strlen(  a.Autor_ ) + 1 ];
  strcpy( Autor_,  a.Autor_ );

  Redakteur_ = new char[ strlen(  a.Redakteur_ ) + 1 ];
  strcpy( Redakteur_,  a.Redakteur_ );

  Seite_ = new char[ strlen(  a.Seite_ ) + 1 ];
  strcpy( Seite_,  a.Seite_ );

  Ausgabe_ = new char[ strlen(  a.Ausgabe_ ) + 1 ];
  strcpy( Ausgabe_,  a.Ausgabe_ );
  
  Jahrgang_ = new char[ strlen(  a.Jahrgang_ ) + 1 ];
  strcpy( Jahrgang_,  a.Jahrgang_ );
  
  Querverweise_ = new char[ strlen(  a.Querverweise_ ) + 1 ];
  strcpy( Querverweise_,  a.Querverweise_ );

  Schlagwoerter_ = new char[ strlen(  a.Schlagwoerter_ ) + 1 ];
  strcpy( Schlagwoerter_,  a.Schlagwoerter_ );

  Zeitschrift_ = a.Zeitschrift_;

  Rating_ = a.Rating_;

  //  initVars()
  // diesen Entry mit Werten aus Entry a fuellen
  //  set(a.Titel(), a.Untertitel(), a.Autor(), a.Redakteur(), 
  //      a.Seite(), a.Ausgabe(), a.Zeitschrift(), a.Jahrgang(), 
  //      a.Querverweise(), a.Schlagwoerter());
}

/* Zuweisungsoperator */
Entry& Entry::operator=(Entry const & a)
{
  if ( &a != this ) {
    // diesen Entry mit Werten aus Entry a fuellen
    set( a.Titel(), a.Untertitel(), a.Autor(), a.Redakteur(), 
	a.Seite(), a.Ausgabe(), a.Zeitschrift(), a.Jahrgang(), 
	a.Querverweise(), a.Schlagwoerter(), a.Rating() );
  }

  return *this;
}

// Destruktor: Speicher der Variablen wieder freigeben
Entry::~Entry()
{

  // wenn sie auf 0 gesetzt sind, dann kann man auch ohne bedenken freigeben
  delete Titel_;
  delete Untertitel_;
  delete Autor_;
  delete Redakteur_;
  delete Seite_;
  delete Ausgabe_;
  delete Jahrgang_;
  delete Querverweise_;
  delete Schlagwoerter_;
}



// Eintraege werden auf einheitliches Format gebracht:
// - keine Umlaute
// - keine Fuehrenden/Abschliessenden Spaces
// Setzen aller Elemente
void Entry::set(char const * aTitel, char const * aUntertitel, char const * aAutor, 
		char const * aRedakteur, char const * aSeite, char const * aAusgabe, 
		MagazinTyp aZeitschrift, char const * aJahrgang, char const * aQuerverweise, 
		char const * aSchlagwoerter, int aRating)
{
  Titel( aTitel );
  Untertitel( aUntertitel );
  Autor( aAutor );
  Redakteur( aRedakteur );
  Seite( aSeite );
  Ausgabe( aAusgabe );
  Zeitschrift( aZeitschrift );
  Jahrgang( aJahrgang );
  Querverweise( aQuerverweise );
  Schlagwoerter( aSchlagwoerter );
  Rating( aRating );
}


#ifdef auskommentiert
// convertiert OriginStr in ConvStr
long Entry::PrepareTheString( char * ConvStr, char const * OriginStr, bool cs )
{
  char * p;
  
  for ( p = ConvStr; *OriginStr; p++, OriginStr++)
    {
      if (!cs) *p = tolower( *OriginStr );
      if ( *OriginStr < '0' )  // Satzzeichen neutralisieren
	*p = ' ';
      else
	switch( (unsigned char) *p )
	  {
	  case ':': *p = ' '; break;
	  case ';': *p = ' '; break;
	  case '<': *p = ' '; break;
	  case '>': *p = ' '; break;
	  case '=': *p = ' '; break;
	  case '?': *p = ' '; break;
	  case '[': *p = ' '; break;
	  case ']': *p = ' '; break;
	  }
    } // for
  *p = '\0';
  
  return( p - ConvStr );  // Laenge
}

// Fuehrt fuzzy suche aus
// threshold gibt an, ueber wieviel Prozent der Match seien muss, damit TRUE 
// zurueckgegeben wird
// testet, ob SearchStr in TextBuffer ist
bool Entry::fuzzy( const char * TextBuffer, const char * SearchStr, bool cs, int threshold )
{

}

#endif // auskommentiert

// Case insensitve strstr
// true - gefunden
// false - nichtgefunden
inline bool Entry::strncasestr( char const * s1, char const * s2, int n ) const
{
  bool ret = false;

  while (*s1) {
    if (strncasecmp(s1, s2, n) == 0) { // wenn gefunden, dann abbrechen
      ret = true;
      break;
    }
    s1++;
  }
  
  return ret;
}

// TriGram Match durchfuehren
// Textsensitive Suche
// SearchLen ist die Laenge des <Text>
// Rueckgabewert: Prozentwert, je hoeher, desto besser
// Der Algorithmus basiert auf einen Artikel in der c't 4/97 "Text-Detektor"
// (uebrigens wurde der Artikel auch mit KHeisePro gesucht :-)
// Die NGramLen ist erstmal auf 3 festgelegt .. bei "2" gibt es wohl zuviele Fehl-Meldungen
inline int Entry::NGramMatch( const char * Text, const char * SearchStr, int SearchLen, int NGramLen )
{
  static char NGram[5];
  static int NGramCount, MaxMatch;
  static int i, Count;

  NGram[NGramLen] = '\0';
  NGramCount = SearchLen - NGramLen + 1;

  for ( i = 0, Count = 0, MaxMatch = 0; i < NGramCount; i++ )
    {
      memcpy( NGram, &SearchStr[i], NGramLen );
      MaxMatch += NGramLen;
      if ( strncasestr(Text, NGram, NGramLen) ) Count++;
    } // for

  return (int) (100.0 * (float) Count * (float) NGramLen / (float) MaxMatch);
}


// Quasi strstr funktion
// beruecksichtigt Case <cs>
// true -> s1 enthaelt s2
// false -> eben nicht
inline bool Entry::contain(const char * s1, const char * s2, bool cs, int rating )
{
  int len = strlen( s2 );
  int foundRating = -1;

  if ( len == 0 )  // leerstring ist nirgendwo drin
    return false;

  // Textsensitive Suche gefordert?
  if ( rating > 0 ) {
    // Rating feststellen
    foundRating = NGramMatch( s1, s2, len, 3 );
    // Nur das hoechste Rating speichern
    if ( foundRating > Rating() )
      Rating( foundRating );
    // true zurueckgeben, wenn foundRating groesser / gleich rating
    return foundRating > rating;
  }


  bool ret = false;

  while (*s1) {
    if (cs) {
      // Case sensitive
      if (strncmp(s1, s2, len) == 0) { // wenn gefunden, dann abbrechen
	ret = true;
	break;
      }
    } else  {
      // Case insensitive
      if (strncasecmp(s1, s2, len) == 0) { // wenn gefunden, dann abbrechen
	ret = true;
	break;
      }
    }
    s1++;
  }

  return ret;
    
}

// enthaelt der String <s1>  alle / oder einen String aus <slist>
// ( abhaenging vom mode <m> )
// true - gefunden
// false - nicht
inline bool Entry::multiContain( char const * s, QStrList *slist, bool cs, bool m, int rating )
{
  bool ret;
  if ( m ) { // AND Verknuepfung
    ret = true;
    // alle strings muessen drin sein
    for (const char* f = slist->first(); f && ret; f = slist->next()) {
      ret &= contain( s, f, cs, rating );
    }
  } else { // OR VERKNUEPFUNG
    ret = false;
    // min ein strings muss drin sein
    for (const char* f = slist->first(); f && !ret; f = slist->next()) {
      ret |= contain( s, f, cs, rating );
    }
  }
  
  return ret;
}

// Ist eine Zahl aus <slist> in <s>?
// immer OR verknuepfung
inline bool Entry::multiNumber( char const * s, QStrList *slist )
{
  bool ret = false;
  // min eine Nummer muss drin sein
  for (const char* f = slist->first(); f && !ret; f = slist->next()) {
    ret |=  ( atoi( f ) == atoi( s ) );
  }

  return ret;
}



/* Vergleicht diesen Entry  mit der SuchMaske */
/* Case beachten (in SearchMask definiert) */
/* ggf. Fuzzy Vergleich implementieren */
// Umlaute: Wurden bereites beim Anlegen vom Entry convertiert
// Ich gehe davon aus, dass alle Felder im Datensatz gesetzt sind.
// muss ggf. noch angepasst werden, wenn ein Feld nicht gesetzt ist.
// Rueckgabewerte: GLEICH , UNGLEICH  (enum Vergleich)
int Entry::compare(SearchMask const & s)
{
  // platz fuer den Grossen suchstring reservieren
  // + 2 fuer die 2 eingefuegten leerzeichen
  // + 1 fuer die abschliessende Null
  // static, damit es nicht bei jedem prozeduraufruf neu angelegt werden muss
  static char longString[TitelMAX + UntertitelMAX + SchlagwoerterMAX + 3];

  // CaseSensitivity ?
  bool cs = s.isCaseSensitive();

  // zuerst Zeitschriftentyp vergleichen
  // Man kann als Maske ja mehrere angeben, daher auf UND Verkn. pruefen
  if ( !(Zeitschrift() & s.Zeitschrift()) )
    return UNGLEICH;

  // Jahrgang, wenn gegeben und vorhanden, vergleichen
  if ( !isEmpty(s.Jahrgang()) ){
    if ( !multiNumber( Jahrgang(), s.JahrgangList ) )
      return UNGLEICH;
  }

  // Ausgabe, wenn gegeben, vergleichen
  if (!isEmpty( s.Ausgabe() ) ) {
    if ( !multiNumber( Ausgabe(), s.AusgabeList ) )
	return UNGLEICH;
  }


  // Redakteur, wenn gegeben, vergleichen 
  // keine ODER Verknuepfung implementiert.
  // im s.Redakteur darf nur ein name stehen
  if (!isEmpty( s.Redakteur() ) )
    if ( !multiContain( Redakteur(), s.RedakteurList, cs, s.RedakteurMode(), s.Rating() ) )
      return UNGLEICH;

  // Autor, wenn gegeben, vergleichen 
  // dito, siehe oben
  if (!isEmpty( s.Autor() ) )
    if ( !multiContain( Autor(),  s.AutorList, cs, s.AutorMode(), s.Rating() ) )
      return UNGLEICH;

  // Hauptsuche: Schlagwoerter
  if ( !isEmpty( s.Schlagwoerter() ) ) {
    // Aus Schlagwoerter, Titel, Untertitle EINEN String mit Leerzeichen machen
    // sonst wird bei UND Verknuepfungen nur ein Treffer gemeldet, 
    // wenn alle Woerter in einem von den drei Feldern vorkommen
    
    // Reste des alten Buffers loeschen
    longString[0] = '\0';
    
    // alle Strings zusammenhaengen
    strcat(longString, Schlagwoerter() );
    strcat(longString, " " );
    strcat(longString, Titel() );
    strcat(longString, " " );
    strcat(longString, Untertitel() );
    
    if ( multiContain( longString, s.SchlagwoerterList, cs, s.SchlagwoerterMode(), s.Rating() ) ) {
      return GLEICH;  
    } else {
      return UNGLEICH;
    }
  }
  
  return GLEICH;
}

// Fuehrt die Convertiertung / Vereinheitlichung von QString a durch
// 1. Umlaute , Scharfes S werden nach Ae, Ue, ss usw .. gewandelt
// 2. Leerzeichen am Anfang / Ende werden rausgenommen
// 3. umwandlung aller Zeichen in kleinbuchstaben, wenn cs = false
//    dadurch werden alle Strings kleingemacht
// liefert Zeiger auf statischen Buffer zurueck, MUSS also kopiert werden!!
char const * Entry::convert( char const * a )
{
  // Rueckgabebuffer, Maximal fuer Schlagwoerter (254 Zeichen)
  static char buffer[254 * 2 + 1];

  // Brauchen einen Zeiger darauf
  char * buf = buffer;

  // Abkuerzung, wenn Null-String
  if (*a == '\0') {
    return a;
  }

  char * start = buf; // fuer die Rueckgabe

  // Leerzeichen am Anfang ueberspringen
  while (*a && isspace(*a))
    a++;

  while (*a) {
    switch(*a) {
      // Zusatz fuer \'a (in a la carte)
    case '\205': *buf++ = 'a'; break;
    case '\204':
    case '': *buf++ = 'a'; *buf++ = 'e'; break;
    case '\216':
    case '': *buf++ = 'A'; *buf++ = 'e'; break;
    case '\224':
    case '': *buf++ = 'o'; *buf++ = 'e'; break; 
    case '\231':
    case '': *buf++ = 'O'; *buf++ = 'e'; break;
    case '\201':
    case '': *buf++ = 'u'; *buf++ = 'e'; break;
    case '\232':
    case '': *buf++ = 'U'; *buf++ = 'e'; break;
    case '\236': 
    case '\341':
    case '': *buf++ = 's'; *buf++ = 's'; break;
    default:  *buf++ = *a; break;
    }
    a++; // Weiter im Quellstring
  }
  // Quellstring ist zuende, jetzt so weit zurueckgehen, wie leerzeichen

  do {
    --a;
    --buf;
  } while (((buf + 1)  > start) && isspace(*a));

  buf++;
  // Abschliessendes \0 anhaengen
  *buf = '\0';


  // Buffer ueber "start" zurueckgeben
  return start;
}



// Einen Entry aus der (schon offenen ) Datei lesen
// Datenformat des Records:
/*
  1. Zeile: Titel, 80 Zeichen
  2. Zeile: Untertitel, 80 Zeichen
  3. Zeile: Autor(en), 80 Zeichen
  4. Zeile: Redakteur, 3 Zeichen
  5. Zeile: Seite, 3 Zeichen
  6. Zeile: Ausgabe, 2 Zeichen
  7. Zeile: ZeitschriftenTyp, 1 Zeichen, Jahrgang 2 Zeichen
  8. Zeile: Querverweise, 80 Zeichen
  9. Zeile: Schlagwoerter, 254 Zeichen
  */

bool Entry::read( FILE * in )
{
  static char line[300];
  static char buf[260];



  // Ueberall: Fehlerabfragen


  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 80 ); buf[80] = '\0';
  // printf( "Titel: >>%s<<\n", buf );
  Titel( buf );
  // printf( "Titel: >>%s<<\n", Titel() );
  
  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 80 ); buf[80] = '\0';
  // printf( "Untertitel: >>%s<<\n", buf );
  Untertitel( buf );
  // printf( "Untertitel: >>%s<<\n", Untertitel() );
  
  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 80 ); buf[80] = '\0';
  // printf( "Autor: >>%s<<\n", buf );
  Autor( buf );
  // printf( "Autor: >>%s<<\n", Autor() );

  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 3 ); buf[3] = '\0';
  // printf( "Redakteur: >>%s<<\n", buf );
  Redakteur( buf );
  // printf( "Redakteur: >>%s<<\n", Redakteur() );
  
  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 3 ); buf[3] = '\0';
  // printf( "Seite: >>%s<<\n", buf );
  Seite( buf );
  // printf( "Seite: >>%s<<\n", Seite() );
  
  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 2 ); buf[2] = '\0';
  // printf( "Ausgabe: >>%s<<\n", buf );
  Ausgabe( buf );
  // printf( "Ausgabe: >>%s<<\n", Ausgabe() );

  MagazinTyp t = CT;

  if ( !fgets( line, 260, in ) ) return false;

  int c = line[0];
  switch ( tolower(c) ) {
  case 'c' : t = CT; break;
  case 'i' : t = IX; break;
  case 'e' : t = EL; break;
  case 'g' : t = GW; break;
  }
  Zeitschrift( t );

  strncpy( buf, &line[1], 2 );  buf[2] = '\0'; 
  // printf( "Typ/Jahrgang: >>%c<< >>%s<<\n", c, buf );
  Jahrgang( buf );
  
  if ( !fgets( line, 260, in ) ) return false;
  strncpy( buf, line, 80 );  buf[80] = '\0';
  // printf( "Querverweise: >>%s<<\n", buf );
  Querverweise( buf );
  // printf( "Querverweise: >>%s<<\n", Querverweise() );
  
  // 300 Zeichen lesen, man kann ja nie wissen
  if ( !fgets( line, 300, in ) ) return false;
  strncpy( buf, line, 254 );  buf[254] = '\0';
  // printf( "Schlagwoerter: >>%s<<\n", buf );
  Schlagwoerter( buf );
  // printf( "Schlagwoerter: >>%s<<\n", Schlagwoerter() );

  return true;
}


// Die allozierten Speicher den Stringroessen anpassen
void Entry::shrinkToFit( void )
{
  char *tmp; 

  tmp = new char[ strlen( Titel_ ) + 1 ];
  strcpy( tmp, Titel_ );
  delete Titel_;
  Titel_ = tmp;

  tmp = new char[ strlen( Untertitel_ ) + 1 ];
  strcpy( tmp, Untertitel_ );
  delete Untertitel_;
  Untertitel_ = tmp;

  tmp = new char[ strlen( Autor_ ) + 1 ];
  strcpy( tmp, Autor_ );
  delete Autor_;
  Autor_ = tmp;

  tmp = new char[ strlen( Redakteur_ ) + 1 ];
  strcpy( tmp, Redakteur_ );
  delete Redakteur_;
  Redakteur_ = tmp;

  tmp = new char[ strlen( Seite_ ) + 1 ];
  strcpy( tmp, Seite_ );
  delete Seite_;
  Seite_ = tmp;

  tmp = new char[ strlen( Ausgabe_ ) + 1 ];
  strcpy( tmp, Ausgabe_ );
  delete Ausgabe_;
  Ausgabe_ = tmp;
  
  tmp = new char[ strlen( Jahrgang_ ) + 1 ];
  strcpy( tmp, Jahrgang_ );
  delete Jahrgang_;
  Jahrgang_ = tmp;
  
  tmp = new char[ strlen( Querverweise_ ) + 1 ];
  strcpy( tmp, Querverweise_ );
  delete Querverweise_;
  Querverweise_ = tmp;

  tmp = new char[ strlen( Schlagwoerter_ ) + 1 ];
  strcpy( tmp, Schlagwoerter_ );
  delete Schlagwoerter_;
  Schlagwoerter_ = tmp;
}

