// Mode: -*-C++-*-

/*
 *  Name         : entry.h
 *  Aufgabe      : Speicherung eines Eintrages der Registerdatei
 *  Version      : $Revision: 1.13 $
 *  Datum        : $Date: 1998/07/01 14:30:40 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef ENTRY_H
#define ENTRY_H

#include <string.h>
#include <stdio.h>

#include <qstrlist.h>

#include "found.h"

// Wie lang koennen die einzelnen Felder max. werden?
#define  TitelMAX 81
#define  UntertitelMAX 81
#define  AutorMAX 81
#define  RedakteurMAX 4
#define  SeiteMAX 4
#define  AusgabeMAX 3
#define  JahrgangMAX 3
#define  QuerverweiseMAX 81
#define  SchlagwoerterMAX 255

// forw. Decl
// das ist uebrigens eine ganz schoen grosse Sch** mit den Dependencies ... *wuerg*
class SearchMask;

/* Klasse zur Speicherung eines Eintrages aus der 
   Register-Datei */

class Entry {
  
public:
  /* Moegliche Magazin-Typen */
  enum MagazinTyp {CT = 0x1,       // CT 
		   IX = 0x2,       // IX 
		   GW = 0x4,       // Gateway
		   EL = 0x8};      // Elrad

  // Wie ist der Vergleich ausgegangen? 
  enum Vergleich { GLEICH, UNGLEICH };

  Entry();
  ~Entry();
  Entry( Entry const & );
  Entry& Entry::operator=(Entry const & );

  /* Setzen aller Daten eines Eintrages */
  void set(char const * aTitel, char const * aUntertitel, char const * aAutor, 
	   char const * aRedakteur, char const * aSeite, char const * aAusgabe, 
	   MagazinTyp aZeitschrift, char const * aJahrgang, char const * aQuerverweise, 
	   char const * aSchlagwoerter, int aRating);


  //
  // Element Access Funktionen
  //
  char const * Titel() const { return Titel_; }
  char const * Untertitel() const { return Untertitel_; }
  char const * Autor() const { return Autor_; }
  char const * Redakteur() const { return Redakteur_; }
  char const * Seite() const { return Seite_; }
  char const * Ausgabe() const { return Ausgabe_; }
  MagazinTyp Zeitschrift() const { return Zeitschrift_; }
  char const * Jahrgang() const { return Jahrgang_; }
  char const * Querverweise() const { return Querverweise_; }
  char const * Schlagwoerter() const { return Schlagwoerter_; }
  int  const   Rating() const { return Rating_; }

  //
  // Element Modifier Funktionen
  //
  void Titel(char const * a )            { strcpy( Titel_, convert( a ) ); }
  void Untertitel( char const * a )      { strcpy( Untertitel_, convert( a ) ); }
  void Autor( char const * a )           { strcpy( Autor_, convert( a ) ); }
  void Redakteur( char const * a )       { strcpy( Redakteur_, convert( a ) ); }
  void Seite( char const * a )           { strcpy( Seite_, convert( a ) ); }
  void Ausgabe( char const * a )         { strcpy( Ausgabe_, convert( a ) ); }
  void Zeitschrift( MagazinTyp const a ) { Zeitschrift_ = a; }
  void Jahrgang( char const * a )        { strcpy( Jahrgang_, convert( a ) ); }
  void Querverweise( char const * a )    { strcpy( Querverweise_, convert( a ) ); }
  void Schlagwoerter( char const * a )   { strcpy( Schlagwoerter_, convert( a ) ); }
  void Rating( int const r )             { Rating_ = r; }

  // Den Eintrag mit dem Suchkriterium des Typs SearchMask
  //   vergleichen 
  // Rueckabe: GLEICH / UNGLEICH
  // Setzt bei Fehlertoleranter Suche das Rating im Entry
  int compare(const SearchMask & );


  // Einen Entry aus dem (schon offenen ) Stream lesen
  // Rueckgabe: true, Erfolg  false sonst
  bool read( FILE * );

  // Groesse der allozierten Speicher zurechtstutzen
  void shrinkToFit();

  // Wandelt alle Umlaute in dem String a um 
  // und entfernt fuehrende / abschliessende Leerzeichen
  // und gibt ne neue Copie zurueck
  static char const * convert( char const * );

  // Ueberprueft, ob s2 in s1 vorkommt 
  // cs -> CaseSentive (true, false)
  bool contain( const char * s1, const char * s2, bool cs, int rating);

  bool multiContain( const char * , QStrList *, bool cs, bool ANDMode, int rating );
  bool multiNumber( const char * , QStrList * );

  int NGramMatch( const char * Text, const char * Search, int SearchLen, int NGramLen );

  bool strncasestr( char const * s1, char const * s2, int n ) const;
 

private:
  // Variablen mit maximaler Groesse anlegen und auf leer setzen
  void initVars( void );
  char  *Titel_;
  char  *Untertitel_;
  char  *Autor_;
  char  *Redakteur_;
  char  *Seite_;
  char  *Ausgabe_;
  MagazinTyp Zeitschrift_;
  char  *Jahrgang_;
  char  *Querverweise_;
  char  *Schlagwoerter_;
  /// Wert zwischen 0 - 100 , prozentual , Textmatch 
  /// 100 default, -1 -> kein Textsensitive Suche
  int   Rating_;   
  
};

#endif /* ENTRY_H */




