// Mode: -*-C++-*-

/*
 *  Name         : found.h
 *  Aufgabe      : Vektor der gefundenen Eintraege
 *  Version      : $Revision: 1.5 $
 *  Datum        : $Date: 1998/07/01 12:07:26 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef FOUND_H
#define FOUND_H

#include <iostream.h>

// #include "entry.h"
class Entry;

/** Diese Klasse repraesentiert die Eintraege, die mit dem
   Suchkriterium uebereinstimmen.  */
// Kapselt einen vector, falls wir nachher noch mal die interne Darstellung aendern
// wollen
// Vektor enthaelt GANZE deep copies von den Entrys, nicht nur Zeiger
// da ja eh Kopien angelegt werden muessen

class Found
{

public:
  /* Sortieren nach... Auf- und Absteigend */
  enum SortType { NONE,
		  AUSG_UP, 
		  AUSG_DOWN, 
		  JAHR_UP, 
		  JAHR_DOWN,
		  TITEL_UP, 
		  TITEL_DOWN, 
		  TYP_UP, 
		  TYP_DOWN,
                  RATING_UP,
                  RATING_DOWN };
  
  /** Konstruktor: Initialisieren der privaten Variablen */ 
  // SortierDefault: JAHR_UP
  // LastSort: JAHR_UP
  // Vektor initialisieren
  Found() :
    v_(0), sortType(NONE), read_(0), write_(0), size_(0)
  {
    // cout << "Found() -> SortType = " << st << endl;
  }

  /** Destruktor: Aufruf von Reset */
  // allen Speicher & Elemente wieder freigeben
  ~Found() { reset(); }

  /** Sortierungs-Modus setzen */
  void setSortType( SortType const st ) { sortType = st; }
  SortType getSortType( void ) { return sortType; }

  /** Einen Kopie des Eintrags von Typ Entry an das Array anghaenge  */
  // Deep Copy anfuegen
  void append( Entry const &);

  /** Das Array entsprechend dem sortTyp sortieren */
  void sort( SortType );

  /* Anzahl der Elemente, die sich im Array befinden
     zurueckgeben */
  int count() const { return size_; }

  /** Speicher des Arrays freigegeben */
  /* Vor dem Starten einer neuen Suche soll die alte Liste der gefunden Entries 
     geloescht werden */
  // alle Elemente des Vectors loeschen
  void reset();
  
  /** Setzt den aktuellen Index auf {\tt aIndex }.
   * {\tt currentIndex} ist nachher zwischen 0 und {\tt apendIndex} - 1, so dass beim
   * naechsten {\tt get} das Element mit dem Index {\tt aIndex}
   * zurueckgeliefert wird 
   * @param aIndex Index, der gesetzt werden soll
   */
  // Index setzen, von dem als naechstes gelesen wird
  // TODO: Bereichsueberpruefung!!
  void setIndex(int const aIndex) { read_ = aIndex; }

  /** Liefert eine Referenz auf das aktuelle Element zurueck.
   */
  Entry const * get() const { return (v_[read_]); }
  
  /* Index auf den naechsten setzten */
  /* true - erfogreich
     false - eben nicht */
  bool next() { return ( read_ == size_ - 1 ) ? false : (read_++ , true); }

  /* Index auf den vorherigen setzten */
  /* true - erfogreich
     false - eben nicht */
  bool prev() { return ( read_ == 0 ) ? false : (read_--, true); }
  

private:
  // Vektor:
  Entry ** v_;
  // Wonach ist das Feld sortiert?
  SortType sortType; 
  // Wonach WAR das Feld sortiert?
  // SortType lastSortType;
  
  // Merken, wo als naechstes gelesen wird
  int read_;
  // Merken, wo als naechstes geschrieben wird
  int write_;
  // Anzahl der Elemente
  int size_;

  // Typedef, um einfacher den Zeiger auf die Comparefunktion deklarieren zu koennen
  typedef  int (Found::*cmpFunc)(Entry const *, Entry const *);
  
  int cmpJahrUp( Entry const *, Entry const * );
  int cmpJahrDown( Entry const *, Entry const * );
  int cmpAusgUp( Entry const *, Entry const * );
  int cmpAusgDown( Entry const *, Entry const * );
  int cmpTitelUp( Entry const *, Entry const * );
  int cmpTitelDown( Entry const *, Entry const * );
  int cmpTypUp( Entry const *, Entry const * );
  int cmpTypDown( Entry const *, Entry const * );
  int cmpRatingUp( Entry const *, Entry const * );
  int cmpRatingDown( Entry const *, Entry const * );
  void q_sort( int, int, int (Found::*)(Entry const *, Entry const *));
  void swap (Entry **, Entry **);

  // ElementReihenfolge umdrehen
  void reverse( void );


};

#endif /* FOUND_H */



