/*
 *  Name         : mainwidget.cpp
 *  Aufgabe      : Haupt-Widget von KHeisePro
 *  Version      : $Revision: 1.24 $
 *  Datum        : $Date: 1999/02/08 22:01:27 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdio.h>

#include <qtablevw.h>
#include <qfontmet.h>
#include <qarray.h>
#include <qpopmenu.h>
#include <qstring.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qframe.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qlist.h>
#include <qcursor.h>
#include <qsize.h>
#include <qtabbar.h>
#include <qtooltip.h>

#include <kapp.h>
#include <ktopwidget.h>
#include <kmenubar.h>
#include <kpopmenu.h>
#include <ktoolbar.h>
#include <kstatusbar.h>
#include <kiconloader.h>

#ifdef HAVE_KKEYCONF_H
#include <kkeyconf.h>
#else
#include <kaccel.h>
#endif

#include <ktablistbox.h>

#include "found.h"
#include "entry.h"
#include "mainwidget.h"
#include "toplevel.h"



/* Konstruktor: die beiden Teil-Widgets 
   FoundList und ShowEntry erstellen */
MainWidget::MainWidget( QWidget *parent = 0, const char *name = 0 )
  : QWidget( parent, name )
{
#ifndef HAVE_KKEYCONF_H
  /* KAccel des Parents uebernehemn */
  Accel = new KAccel( this );//((TopLevel *)parent)->getAccel();
#endif

  /* Top-Layout anlegen */
  topLayout = new QVBoxLayout( this, 0 );

  createTabBar();
  createFoundList();
  createShowEntry();

  topLayout->activate();

  /* Up- / Down-Buttons zum Anfang disablen */
  ButtonUp->setEnabled( false );
  ButtonDown->setEnabled( false );


#ifndef HAVE_KKEYCONF_H  
  /* So gehts seit dem 1.1-Branch...*/
  Accel->insertItem( "Zeile hoch", "lineup", "Up" );
  Accel->insertItem( "Zeile runter", "linedown", "Down" );
  Accel->insertItem( "Seite hoch", "pageup", "PageUp" );
  Accel->insertItem( "Seite runter", "pagedown", "PageDown" );

  Accel->connectItem( "lineup", this, SLOT(moveUp()) );
  Accel->connectItem( "linedown", this, SLOT(moveDown()) );
  Accel->connectItem( "pageup", this, SLOT(move10Up()) );
  Accel->connectItem( "pagedown", this, SLOT(move10Down()) );

  Accel->readSettings();
  updateMenuAccel();
  
#else
  /* Acceleratoren fuer das Mainwidget erzeugen */
  kKeys->addKey( "Zeile hoch", "Up" );
  kKeys->addKey( "Zeile runter", "Down" );
  kKeys->addKey( "Seite hoch", "PageUp" );
  kKeys->addKey( "Seite runter", "PageDown" );

  QString widgetName = this->name();

  kKeys->registerWidget( widgetName, this);

  kKeys->connectFunction( widgetName, "Zeile hoch", this, SLOT(moveUp()) );
  kKeys->connectFunction( widgetName, "Zeile runter", this, SLOT(moveDown()) );
  kKeys->connectFunction( widgetName, "Seite hoch", this, SLOT(move10Up()) );
  kKeys->connectFunction( widgetName, "Seite runter", this, SLOT(move10Down()) );
#endif /* HAVE_KKEYCONF_H */

  haveCtrom = true;
}


/* Erzeugen des TabBars von MainWidget */
void MainWidget::createTabBar( void )
{
  tabBar = new QTabBar( this, "TabBar" );
  tabBar->setMinimumHeight( 30 ); //tabBar->sizeHint().height() );
  connect( tabBar, SIGNAL(selected(int)), this, SIGNAL(tabChanged(int)) );
  
  tab1 = new QTab;
  tab2 = new QTab;
 
  /* Tab fuer normale Suche */
  tab1->label = "Suche";
  tab1->enabled = true;
  tab1->id = 1;

  /* Tab fuer Vormerkungen */
  tab2->label = "Vormerkungen";
  tab2->enabled = true;
  tab2->id = 2;

  searchTabID = tabBar->addTab( tab1 );
  markedTabID = tabBar->addTab( tab2 );


  // Layouten
  topLayout->addWidget( tabBar, 0 );
}



/* Erzeugen des ShowEntry-Teils von Mainwidget */
void MainWidget::createShowEntry( void )
{
  /* Layout erstellen */
  QHBoxLayout *lowerBox = new QHBoxLayout( 10 );
  topLayout->addSpacing( 5 );
  topLayout->addLayout( lowerBox, 0 );
  topLayout->addSpacing( 5 );

  QGridLayout *leftBox = new QGridLayout( 5, 6, 5 );
  lowerBox->addLayout( leftBox, 1 );
  
  leftBox->setColStretch( 1, 1 );
  leftBox->setColStretch( 3, 1 );
  leftBox->setColStretch( 5, 1 );


  /* Alle Labels und deren Beschreibungen anlegen */

  /* Feld fuer den Eintrag "Titel" */
  Titel = new QLabel( this, "Titel_lab" );
  Titel->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Titel->setMinimumHeight( Titel->fontMetrics().height() + 10 );
  desc1 = new QLabel( Titel, "Titel:", this );
  desc1->setMinimumSize( desc1->sizeHint() );
  // Layouten
  leftBox->addWidget( desc1, 0, 0 );
  leftBox->addMultiCellWidget( Titel, 0, 0, 1, 5 );
  
  /* Feld fuer den Eintrag "Untertitel" */
  Untertitel = new QLabel( this, "UTitel_lab" );
  Untertitel->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Untertitel->setMinimumHeight( Untertitel->fontMetrics().height() + 10 );
  desc2 = new QLabel( Untertitel, "Untertitel:", this );
  desc2->setMinimumSize( desc2->sizeHint() );
  // Layouten
  leftBox->addWidget( desc2, 1, 0 );
  leftBox->addMultiCellWidget( Untertitel, 1, 1, 1, 5 );


  /* Feld fuer den Eintrag "Autor" */
  Autor = new QLabel( this, "Autor_lab" );
  Autor->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Autor->setMinimumHeight( Autor->fontMetrics().height() + 10 );
  desc3 = new QLabel( Autor, "Autor(en):", this );
  desc3->setMinimumSize( desc3->sizeHint() );
  // Layouten
  leftBox->addWidget( desc3, 2, 0 );
  leftBox->addMultiCellWidget( Autor, 2, 2, 1, 3 );


  /* Feld fuer den Eintrag "Redakteur" */
  Redakteur = new QLabel( this, "Readakt_lab" );
  Redakteur->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Redakteur->setMinimumHeight( Redakteur->fontMetrics().height() + 10 );
  desc4 = new QLabel( Redakteur, "Redakteur:", this );
  desc4->setMinimumSize( desc4->sizeHint() );
  // Layouten
  leftBox->addWidget( desc4, 2, 4 );
  leftBox->addWidget( Redakteur, 2, 5 );


  /* Feld fuer den Eintrag "Seite" */
  Seite = new QLabel( this, "Seite_lab" );
  Seite->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Seite->setMinimumHeight( Seite->fontMetrics().height() + 10 );
  desc5 = new QLabel( Seite, "Seite:", this );
  desc5->setMinimumSize( desc5->sizeHint() );
  // Layouten
  leftBox->addWidget( desc5, 3, 0 );
  leftBox->addWidget( Seite, 3, 1 );


  /* Feld fuer den Eintrag "Ausgabe" */
  Ausgabe = new QLabel( this, "Ausg_lab" );
  Ausgabe->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Ausgabe->setMinimumHeight( Ausgabe->fontMetrics().height() + 10 );
  desc6 = new QLabel( Ausgabe, "Ausgabe:", this );
  desc6->setMinimumSize( desc6->sizeHint() );
  // Layouten
  leftBox->addWidget( desc6, 3, 2 );
  leftBox->addWidget( Ausgabe, 3, 3 );


  /* Feld fuer den Eintrag "Zeitschrift" */
  Zeitschrift = new QLabel( this, "Zeitschr_lab" );
  Zeitschrift->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Zeitschrift->setMinimumHeight( Zeitschrift->fontMetrics().height() + 10 );
  desc7 = new QLabel( Zeitschrift, "Zeitschrift:", this );
  desc7->setMinimumSize( desc7->sizeHint() );
  // Layouten
  leftBox->addWidget( desc7, 3, 4 );
  leftBox->addWidget( Zeitschrift, 3, 5 );


  /* Feld fuer den Eintrag "Verweise" */
  Verweise = new QLabel( this, "Verw_lab" );
  Verweise->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  Verweise->setMinimumHeight( Verweise->fontMetrics().height() + 10 );
  desc8 = new QLabel( Verweise, "Verweise:", this );
  desc8->setMinimumSize( desc8->sizeHint() );
  // Layouten
  leftBox->addWidget( desc8, 4, 0 );
  leftBox->addMultiCellWidget( Verweise, 4, 4, 1, 5 );



  /* Buttons zum hoch- und runterblaettern anlegen */
  KIconLoader *loader = kapp->getIconLoader();

  /* Layoutframes */
  QVBoxLayout *rightBox = new QVBoxLayout( 10 );
  lowerBox->addLayout( rightBox, 0 );
  lowerBox->addSpacing( 5 );

  ButtonUp = new QPushButton( this, "ButUp" );
  QPixmap pm = loader->loadIcon( "arrow_up.xpm" );
  ButtonUp->setPixmap( pm );
  ButtonUp->setFixedSize( QSize( 40, 40 ) );
  // Layouten
  rightBox->addStretch( 1 );
  rightBox->addWidget( ButtonUp, 0 );

  ButtonDown = new QPushButton( this, "ButDwn" );
  pm = loader->loadIcon( "arrow_down.xpm" );
  ButtonDown->setPixmap( pm );
  ButtonDown->setFixedSize( QSize( 40, 40 ) );
  // Layouten
  rightBox->addWidget( ButtonDown, 0 );
  //  rightBox->addStretch( 1 );

  connect( ButtonUp, SIGNAL(clicked()), this, SLOT(moveUp()) );
  connect( ButtonDown, SIGNAL(clicked()), this, SLOT(moveDown()) );

  
  /* Button zum Oeffnen des KFM mit dem entsprechenden 
     Artikel anlegen */
  ButtonCD = new QPushButton( "open", this, "ButCD" );
  pm = loader->loadIcon( "cdrom.xpm" );
  ButtonCD->setPixmap( pm );
  ButtonCD->setFixedSize( QSize( 40, 40 ) );
  // Tooltip fuer den Button 
  QToolTip::add( ButtonCD, "Oeffne Artikel in KFM" );
  // Layouten
  rightBox->addWidget( ButtonCD, 0 );
  rightBox->addStretch( 1 );

  connect( ButtonCD, SIGNAL(clicked()), this, SIGNAL(openArticleClicked()) );
}


/* Erzeugen des FoundList-Teils von Mainwidget */
void MainWidget::createFoundList( void )
{
  /* Listbox anlegen ... */
  ListBox = new KTabListBox( this, "Listbox", 6, 0 );
  /* ... und in 4 Spalten unterteilen */
  ListBox->setColumn( 0, "Rate", 0, KTabListBox::TextColumn );
  ListBox->setColumn( 1, "Titel", 200, KTabListBox::TextColumn );
  ListBox->setColumn( 2, "Untertitel", 200, KTabListBox::TextColumn );
  ListBox->setColumn( 3, "Typ", 40, KTabListBox::TextColumn );
  ListBox->setColumn( 4, "Ausg.", 40, KTabListBox::TextColumn );
  ListBox->setColumn( 5, "Jahr", 40, KTabListBox::TextColumn );
  
  // Layouten
  topLayout->addWidget( ListBox, 1 );

  /* Separator-Char ist /t */
  ListBox->setSeparator( '\t' );  

  /* KontextMenue der Listbox erstellen */
  createKontextMenus();

  /* Connection: Wenn auf eine Zeile der Listbox geklickt wird,
     dann wird ueberprueft, ob einer der Up / Down Buttons
     disabled werden muss */
  connect( ListBox, SIGNAL(highlighted(int, int)), this, SLOT(setCurrent(int, int)) ); 

  /* Connection: Click auf auf den Header wird durchgreicht */
  connect( ListBox, SIGNAL(headerClicked(int)), this, SIGNAL(headerClicked(int)) );

  /* Connection: Wenn die rechte Maustaste gedrueck wird, dann
     wird das Kontext-Menue angezeigt */
  connect( ListBox, SIGNAL(popupMenu(int,int)), this, SLOT(doKontextMenu()) );
}


/* Resize-Funktion des Main-Widgets */
/* Hier werden nur die Spalten der ListBox neu gesetzt,
   das eigentliche Resizen uebernimmt das QBoxLayout */
void MainWidget::resizeEvent( QResizeEvent * )
{
  int charWidth = QFontMetrics( ListBox->font() ).maxWidth();
  int xSize;

  /* Die Spalten der Listbox werden folgendermassen unterteilt:
     Die 1. Spalte ist 4,5 Zeichen breit, die 4., 5. und 6. Spalte sind jeweils 3,5 Zeichen breit,
     die 2. und 3. Spalte teilen sich den Rest in Verhaehlnis 1:1 */
  if( showRating )
    {
      xSize = ListBox->width() - charWidth * 15;
      ListBox->setColumnWidth( 0, charWidth * 9 / 2 );
    }
  else
    {
      xSize = ListBox->width() - charWidth * 21 / 2;
      ListBox->setColumnWidth( 0, 0 );
    }
  ListBox->setColumnWidth( 1, xSize / 2 );
  ListBox->setColumnWidth( 2, xSize / 2 );
  ListBox->setColumnWidth( 3, charWidth * 7 / 2 );
  ListBox->setColumnWidth( 4, charWidth * 7 / 2 );
  /* Letzte Spalte bis zum Rand des Fensters */
  ListBox->setColumnWidth( 5, ListBox->width() - ListBox->columnWidth(0) - ListBox->columnWidth(1) 
			   - ListBox->columnWidth(2) - ListBox->columnWidth(3) );
  /* Aenderungen sichtbar machen */
  ListBox->repaint();

  /* Current sichtbar machen */
  ListBox->setTopItem( ListBox->currentItem() );
}



/* Funktion zum loeschen aller Eintrage in den Feldern */
void MainWidget::clearShowEntry( void )
{
  /* Alle Felder mit einem Leerstring auffuellen */
  Titel->setText("");
  Untertitel->setText("");
  Autor->setText("");
  Redakteur->setText("");
  Seite->setText("");
  Ausgabe->setText("");
  Zeitschrift->setText("");
  Verweise->setText("");
}


/* Funktion zum Anzeigen eines Entry-Objektes */
void MainWidget::updateShowEntry( const Entry *Item )
{
  /* Soll das ShowEntry nur geloescht werden ??
     Item == NULL */
  if( !Item )
    {
      clearShowEntry();
      // Open-Button disablen 
      ButtonCD->setEnabled( false );
      return;
    }
  /* ansonsten... */
  
  /* Strings in Labels setzen */
  Titel->setText( (const char *) Item->Titel() );
  Untertitel->setText( (const char *) Item->Untertitel() );
  Autor->setText( (const char *) Item->Autor() );
  Redakteur->setText( (const char *) Item->Redakteur() );
  Seite->setText( (const char *) Item->Seite() );
  Verweise->setText( (const char *) Item->Querverweise() );

  /* "Ausgabe" und "Jahrgang" in einen String
     wandeln und in das Feld "Ausgage" schreiben */
  QString help;
  help.append( Item->Ausgabe() );
  help.append( " / " );
  help.append( Item->Jahrgang() );
  Ausgabe->setText( (const char *) help );
  
  /* Zeitungstyp in String wandeln */
  switch( Item->Zeitschrift() )
    {
    case (Entry::CT):
      help = "ct";
      break;
    case (Entry::IX):
      help = "ix";
      break;
    case (Entry::GW):
      help = "Gateway";
      break;
    case (Entry::EL):
      help = "Elrad";
      break;
    }
  /* und in Label schreiben */
  Zeitschrift->setText( (const char *) help );

  // Open-Button enablen, falls Ctrom vorhanden
  ButtonCD->setEnabled( haveCtrom );
}




/* Funktion zur erstellung des Kontextmenues von der
   KTabListBox */
void MainWidget::createKontextMenus( void )
{
  kontextMenu = new QPopupMenu();
  clipboardMarkID = kontextMenu->insertItem( "Kopiere markierte Eintrge ins Clipboard", this, SIGNAL(clipboardMarked()) );
  kontextMenu->insertSeparator();
  exportAllID = kontextMenu->insertItem( "Exportiere alle Eintrge...", this, SIGNAL(exportAll()) );
  exportMarkID = kontextMenu->insertItem( "Exportiere markierte Eintrge...", this, SIGNAL(exportMarked()) );
  kontextMenu->insertSeparator();
  printAllID = kontextMenu->insertItem( "Drucke alle Eintrge", this, SIGNAL(printAll()) );
  printMarkID = kontextMenu->insertItem( "Drucke markierte Eintrge", this, SIGNAL(printMarked()) );
  kontextMenu->insertSeparator();
  preAllID = kontextMenu->insertItem( "Alle Eintrge vormerken", this, SIGNAL(addAll()) );
  preMarkID = kontextMenu->insertItem( "Markierte Eintrge vormerken", this, SIGNAL(addMarked()) );
  kontextMenu->insertSeparator();
  remAllID = kontextMenu->insertItem( "Alle Eintrge entfernen", this, SIGNAL(removeAll()) );
  remMarkID = kontextMenu->insertItem( "Markierte Eintrge entfernen", this, SIGNAL(removeMarked()) );
}



/* Hinzufuegen / Aendern der Kontext-Menuepunkte
   -> Acceleratoren werden mit angezeigt */
void MainWidget::updateMenuAccel( void )
{
#ifndef HAVE_KKEYCONF_H
  Accel->changeMenuAccel( kontextMenu, preAllID, "markall" );
  Accel->changeMenuAccel( kontextMenu, preMarkID, "markmarked" );

#else
  changeMenuAccel( kontextMenu, preAllID, "Alle Eintraege vormerken" );
  changeMenuAccel( kontextMenu, preMarkID, "Markierte Eintraege vormerken" );
#endif /* HAVE_KKEYCONF_H */
}


#ifdef HAVE_KKEYCONF_H
/* Zu einem Menuepunkt den Accelerator hinzufuegen */
/* Betrifft nur den Text, der im Menue steht */
void MainWidget::changeMenuAccel ( QPopupMenu *menu, int id,
				 const char *functionName )
{
  QString s = menu->text( id );
  if ( !s ) return;
  
  int i = s.find('\t');
  
  QString k = keyToString( kKeys->readCurrentKey( functionName ) );
  if( !k ) return;
  
  if ( i >= 0 )
    s.replace( i+1, s.length()-i, k );
  else {
    s += '\t';
    s += k;
  }
  
  menu->changeItem( s, id );
}
#endif /* HAVE_KKEYCONF_H */


/* Funktion zum Eintragen einer Liste von gefundenen
   Eintraegen aus einem Found-Objekt */
void MainWidget::fillList( Found *List )
{
  /* Soll die Liste nur geloescht werden ?? */
  if( !List || !List->count() )
    {
      /* Liste loeschen, Buttons disablen */
      clearFoundList();
      ButtonUp->setEnabled( false );
      ButtonDown->setEnabled( false );
      updateShowEntry( 0 );
      return;
    }

  /* ansonsten... */

      
  /* Liste der gefundenen Eintraege auf Anfang setzen */
  List->setIndex( 0 );

  /* ListBox loeschen und AutoUpdate ausschalten */
  ListBox->setAutoUpdate( false );
  clearFoundList();

  
  const Entry *actual;
  char newItem[204];

  QString magTyp;
  QString rating;

  do
    {
      actual = List->get();
      
      /* Zeitschrift-Typ aufbereiten */
      switch( actual->Zeitschrift() )
	{
	case (Entry::CT):
	  magTyp.setStr( "c't" );
	  break;
	case (Entry::IX):
	  magTyp.setStr( "iX" );
	  break;
	case (Entry::EL):
	  magTyp.setStr( "El" );
	  break;
	case (Entry::GW):
	  magTyp.setStr( "Gw" );
	  break;
	}
      
      /* Rating aufbereiten */
      int hlp;
      if( (hlp = actual->Rating()) == -1 )
	hlp = 100;
      rating.setNum( hlp );
      rating += " %";

      /* Gesamte Zeile zusammenstellen */
      sprintf( newItem, "%.10s \t%.90s \t%.90s \t%.10s \t%.10s \t%.10s",
	       (const char *) rating,
	       (const char *) actual->Titel(), (const char *) actual->Untertitel(),
	       (const char *) magTyp,
	       (const char *) actual->Ausgabe(), (const char *) actual->Jahrgang() );

      /* ... und in die ListBox einfuegen */
      ListBox->insertItem( newItem );
    } while( List->next() );

  ListBox->setAutoUpdate( true );
  ListBox->setCurrentItem( 0 );
  ListBox->repaint();
} 


/* Einen Eintrag hoeher springen */
void MainWidget::moveUp( void )
{
  int curItem = ListBox->currentItem();

  /* Wenn oberes Ende der Listbox noch nicht erreicht, dann
     current nach oben verschieben */
  if( curItem > 0 )
    ListBox->setCurrentItem( curItem - 1 );

  /* Inhalt der Listbox so verschieben, dass die aktuelle
     Zeile nie hieher als die oberste sichtbare Zeile ist */
  if( curItem <= ListBox->topItem() || curItem > ListBox->lastRowVisible() )
    ListBox->setTopItem( curItem - 1 );
}


/* Einen Eintrag niedriger springen */
void MainWidget::moveDown( void )
{
  int curItem = ListBox->currentItem();
  int rowAnz = ListBox->count();

  /* Wenn unteres Ende noch nicht erreicht, dann
     current eine Zeile weiter setzen */
  if( curItem < rowAnz - 1 )
    ListBox->setCurrentItem( curItem + 1 );

  /* Inhalt der Listbox so verschieben, dass die
     aktuelle Zeile nie tiefer als die vorletzte sichtbare Zeile ist
     (ausser natuerlich, wenn die aktuelle die letzte ist) */
  if( curItem >= ListBox->lastRowVisible() - 1 )
    ListBox->setTopItem( ListBox->topItem() + 1 );
  if( curItem < ListBox->topItem() )
    ListBox->setTopItem( curItem );
}



/* 10 Eintraege hoeher springen */
void MainWidget::move10Up( void )
{
  /* 10 Eintraege von aktuellen nach oben springen */
  int curItem = ListBox->currentItem() - 10;

  /* Ausgeleichen, falls oberes Ende der Liste erreicht */
  if( curItem < 0 )
    curItem = 0;

  ListBox->setCurrentItem( curItem );

  /* Inhalt der Listbox so verschieben, dass die aktuelle
     Zeile nie hieher als die oberste sichtbare Zeile ist */
  if( curItem <= ListBox->topItem() || curItem > ListBox->lastRowVisible() )
    ListBox->setTopItem( curItem );
}



/* 10 Eintraege niedriger springen */
void MainWidget::move10Down( void )
{
  /* Versuchen, den Current um 10 nach unten zu schieben */
  int curItem = ListBox->currentItem() + 10;
  int lastItem = ListBox->count() - 1;


  if( curItem > lastItem )
    curItem = lastItem;

  ListBox->setCurrentItem( curItem );

  /* Inhalt der Listbox so verschieben, dass die
     aktuelle Zeile nie tiefer als die vorletzte sichtbare Zeile ist
     (ausser natuerlich, wenn die aktuelle die letzte ist) */
  if( curItem >= ListBox->lastRowVisible() )
    ListBox->setTopItem( ListBox->topItem() + 10 );
  if( curItem < ListBox->topItem() )
    ListBox->setTopItem( curItem );
}



/* Listbox loeschen */
void MainWidget::clearFoundList( void )
{
  ListBox->clear();
  ListBox->repaint();
}



/* Current auf line setzen, dabei ueberpruefen
   welche der Up-/ Down-Buttons disabled /enabled werden
   muessen */
void MainWidget::setCurrent( int line, int )
{
  int rowAnz = ListBox->count();

  // cout << "index: " << line << endl;

  /* Neuer Index in den richtigen Grenzen und 
     nicht der current (->Rekursion) ? */
  //  if( (line != ListBox->currentItem()) )   //(line > 0) && (line <= rowAnz - 1) && 
  //    ListBox->setCurrentItem( line );

  emit currentChanged( line );

  /* Ist die neue Zeile die erste ? */
  if( line == 0 )
    /* Up-Button disablen */
    ButtonUp->setEnabled( false );
  else
    /* sonst darf man hochblaettern */
    ButtonUp->setEnabled( true );
    
  /* Ist die neue Ziele die letzte ? */
  if( line == rowAnz - 1 )
    /* Down-Button disablen */
    ButtonDown->setEnabled( false );
  else
    /* Sonst darf man runterblaettern */
    ButtonDown->setEnabled( true );
}


/* "Aufpoppen" des Kontext-Menues von FoundList */
void MainWidget::doKontextMenu( void )
{
  QCursor dummy(0);
  bool enab = ( tabBar->currentTab() == searchTabID );

  /* Menuepunkte entsprechend dem aktuellen Tab
     enablen / disablen */
  kontextMenu->setItemEnabled( preAllID, enab );
  kontextMenu->setItemEnabled( preMarkID, enab );
  kontextMenu->setItemEnabled( remAllID, !enab );
  kontextMenu->setItemEnabled( remMarkID, !enab );

  /* Menue aufpoppen */
  kontextMenu->popup( dummy.pos() );
}


/* Zurueckliefern einer QList, der die Indizes aller
   markierten Eintraege enthaelt */
QList<int> MainWidget::getMarkedItems( void )
{
  QList<int> res;
  for( unsigned int c = 0; c < ListBox->count(); c++ )
    {
      if( ListBox->isMarked( c ) )
	{
	  int *app = new int( c );
	  res.append( app );
	}
    }

  return res;
}


/* Behandlung eines Klicks auf die CheckBox "Fehlertolerant suchen" im
   SearchDialog => zeigen / verstecken der Spalte "Rate" von FoundList */
void MainWidget::handleDoToleranceClicked( bool enab )
{
  showRating = enab;
  resizeEvent( (QResizeEvent *) 0 );
}


#include "mainwidget.moc"
