// Mode: -*-C++-*-

/*
 *  Name         : mainwidget.h
 *  Aufgabe      : Haupt-Widget von KHeisePro
 *  Version      : $Revision: 1.13 $
 *  Datum        : $Date: 1998/12/09 21:30:46 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef MAINWIDGET_H
#define MAINWIDGET_H


#include <qwidget.h>
#include <qlist.h>
#include <ktopwidget.h>
#include <qtabbar.h>
#include <qpushbt.h>

class Entry;
class Found;
class QKeyEvent;
class QPopupMenu;
class QLabel;
class QTabBar;
class QTab;
class QVBoxLayout;
class KTabListBox;

#ifndef HAVE_KKEYCONF_H
class KAccel;
#endif /* HAVE_KKEYCONF_H */

/** Klasse des in KTopLevelWidget eingebetteten
  MainWidgets */
class MainWidget : public QWidget
{
  Q_OBJECT

public:
  /** Konstruktor: Anlegen des Widgets inkl. Menubar, Toolbar, Statusbar*/
  MainWidget( QWidget *parent = 0, const char *name = 0 );

  /** Loeschen des Inhalts */
  void clearShowEntry( void );

  /** Loeschen des gesamten Widgets / der
      gesamten Liste */
  void clearFoundList( void );

  /** Funktion liefert eine QList zurueck, der
      die Indizes aller markierten Eintraege enthaelt */
  QList<int> getMarkedItems( void );

  /** Funktions liefert den aktuellen Tab zurueck */
  int getActTab( void ) { return tabBar->currentTab(); }

  /** Funktion setzt den aktuellen Tab */
  void setActTab( int num ) { tabBar->setCurrentTab( num ); }

  /** Updaten der Accelerator-Strings in den Kontextmenu-Eintraegen */
  void updateMenuAccel( void );


public slots:
  /** Slot zur Anzeige eines neuen
    Eintrages */
  void updateShowEntry( const Entry *Item );


  /** Slot zum Auffuellen der Liste mit dem Ergebnis einer
    neuen Suche oder der neu sortierten Liste */
  void fillList( Found *List );

  /** Nach einem Klick auf "Hochblaettern" */
  void moveUp();

  /** Nach einem Klich auf "Runterblaettern" */
  void moveDown();

  /** Slot zum hochblaettern um 10 Eintraege */
  void move10Up();

  /** Slot zum herunterblaettern um 10 Eintraege */
  void move10Down();

  /** Zum expliziten aktivieren einer bestimmten Zeile
      der Listbox */
  void setCurrent( int, int );

  /** Anzeige des Popup-Menues von KTabListBox */
  void doKontextMenu();

  /** Setzen der Variable "haveCtrom" */
  void setHaveCtrom( bool haveIt ) { haveCtrom = haveIt; }

  /** Enablen / Disablen des Open-Buttons */
  void setOpenButtonEnabled( bool enab ) { ButtonCD->setEnabled( enab ); }

  /** Behandlung eines Clicks auf die CheckBox "Fehlertolerant Suchen"
      im SearchDialog */
  void handleDoToleranceClicked( bool );


signals:
  /** Signal zum Durchreichen eines Klicks auf die
      Titelzeile der Listbox */
  void headerClicked( int );

  /** Signal zum Durchreichen einer Aenderung des
      Current der ListBox */
  void currentChanged( int );

  /** Signal: Markierte exportieren in Kontextmenue ausgewaehlt */
  void exportMarked();

  /** Signal: Alle Eintraege exportieren in Kontextmenue ausegw. */
  void exportAll();

  /** Signal: Markierte drucken in Kontextmenue ausgewaehlt */
  void printMarked();

  /** Signal: Alle Eintraege drucken in Kontextmenue ausgewaehlt */
  void printAll();

  /** Signal: Markierte Eintraege ins Clipboard */
  void clipboardMarked();

  /** Signal: Alle Eintraege in die Vormerkung kopieren */
  void addAll();

  /** Signal: Markierte Eintraege in die Vormerkung kopieren */
  void addMarked();

  /** Signal: Alle Eintraege aus der Vormerkung entfernen */
  void removeAll();
  
  /** Signal: Markierte Eintraege aus der Vormerkung entfernen */
  void removeMarked();

  /** Signal: Der Benutzer hat auf die TabLeiste geklickt */
  void tabChanged( int );

  /** Signal: Auf CD geklickt */
  void openArticleClicked();


protected:
  /** Resize-Event */
  void resizeEvent( QResizeEvent * );


private:
  /** Erstellen des TabBars */
  void createTabBar( void );

  /** Erstellen des ShowEntry-Teils */
  void createShowEntry( void );

  /** Erstellen des FoundList-Teils */
  void createFoundList( void );

  /** Funktions zur Erstellung des KontextMenues */
  void createKontextMenus( void );

#ifdef HAVE_KKEYCONF_H
  /** Hilfsfunktion zu updateMenuAccel */
  void changeMenuAccel ( QPopupMenu *, int ,  const char * );
#endif /* HAVE_KKEYCONF_H */

  /** TabBar fuer die Suche / Vormerkunen */
  QTabBar *tabBar;

  /** Tabulatoren des TabBar */
  QTab *tab1, *tab2;

  /** IDs der Tabs */
  int searchTabID, markedTabID;


  /** Die Listbox */
  KTabListBox *ListBox;
  
  /** Kontextmenue der Listbox */
  QPopupMenu *kontextMenu;

  /** IDs der Kontext-Menuepunkte */
  int exportAllID, exportMarkID, printAllID, printMarkID, preAllID;
  int preMarkID, remAllID, remMarkID, clipboardMarkID;

#ifndef HAVE_KKEYCONF_H
  /** Acceleratoren des Kontextmenues */
  KAccel *Accel;
#endif /* HAVE_KKEYCONF_H */

  /** Felder zur Anzeige eines Entrys */
  QLabel *Titel;
  QLabel *Untertitel;
  QLabel *Autor;
  QLabel *Redakteur;
  QLabel *Seite;
  QLabel *Ausgabe;
  QLabel *Zeitschrift;
  QLabel *Verweise;

  QLabel *desc1;
  QLabel *desc2;
  QLabel *desc3;
  QLabel *desc4;
  QLabel *desc5;
  QLabel *desc6;
  QLabel *desc7;
  QLabel *desc8;

  /* topLayout wird in zwei Funktionen benutzt, daher global */
  QVBoxLayout *topLayout;

  /** Buttons zum blaettern */
  QPushButton *ButtonUp;
  QPushButton *ButtonDown;


  /** Bool-Variable: ct-roms da oder nicht da */
  bool haveCtrom;

  /** Button zum oeffnen des Artikels auf der CD */
  QPushButton *ButtonCD;

  /** Bool zum Speichern, ob Rating-Spalte angezeigt werden
      soll oder nicht */
  bool showRating;
};



#endif /* MAINWIDGET_H */





