/*
 *  Name         : searchmask.cpp
 *  Aufgabe      : Speicherung der Suchkriterien
 *  Version      : $Revision: 1.7 $
 *  Datum        : $Date: 1998/07/01 09:32:50 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <string.h>

#include <qstrlist.h>

#include <iostream.h>

#include "searchmask.h"


// Fuern einfacheren Test auf "leer"
#define isEmpty(c) (c[0] == '\0')


// Constructor
SearchMask::SearchMask( bool cs = false ) : 
  caseSensitive( cs ) 
{
  // cout << "SearchMask() Anfang" << endl;
  Schlagwoerter_ = new char[1];
  *Schlagwoerter_ = '\0';
  Autor_ = new char[1];
  *Autor_ = '\0';
  Redakteur_ = new char[1];
  *Redakteur_ = '\0';
  Ausgabe_ = new char[1];
  *Ausgabe_ = '\0';
  Jahrgang_ = new char[1];
  *Jahrgang_ = '\0';
  
  // Verknuepfung default auf AND
  SchlagwoerterMode( true );
  AutorMode( true );
  RedakteurMode( true );
  
  // Listen als 'leer' markieren
  SchlagwoerterList = 0;
  AutorList         = 0;
  JahrgangList      = 0;
  RedakteurList     = 0;
  AusgabeList       = 0;
  
  // cout << "SearchMask() Ende" << endl;
}

// Destructor
SearchMask::~SearchMask()
{
  // cout << "~SearchMask() Anfang" << endl;

  delete[] Schlagwoerter_; 
  delete[] Jahrgang_;
  delete[] Autor_; 
  delete[] Redakteur_; 
  delete[] Ausgabe_;

  delete SchlagwoerterList;
  delete AutorList;
  delete JahrgangList;
  delete RedakteurList;
  delete AusgabeList;

  // cout << "~SearchMask() Ende" << endl;
}

// Copyconstructor
SearchMask::SearchMask( SearchMask const & rhs )
{
  // cout << "SearchMask CopyConstructor" << endl;

  if ( &rhs != this ) {

    Schlagwoerter_ = new char[strlen(rhs.Schlagwoerter()) + 1];
    strcpy( Schlagwoerter_, Entry::convert( rhs.Schlagwoerter() ) ); 
    
    Jahrgang_= new char[strlen(rhs.Jahrgang()) + 1];
    strcpy( Jahrgang_, Entry::convert( rhs.Jahrgang() ) );
    
    Autor_= new char[strlen(rhs.Autor()) + 1]; 
    strcpy( Autor_, Entry::convert( rhs.Autor() ) );
    
    Redakteur_= new char[strlen(rhs.Redakteur()) + 1]; 
    strcpy( Redakteur_, Entry::convert( rhs.Redakteur() ) ); 
    
    Ausgabe_= new char[strlen(rhs.Ausgabe()) + 1];
    strcpy( Ausgabe_, Entry::convert( rhs.Ausgabe() ) ); 
    
    setCaseSensitive( rhs.isCaseSensitive() );
    Zeitschrift( rhs.Zeitschrift() );

    SchlagwoerterMode( rhs.SchlagwoerterMode() );
    AutorMode( rhs.AutorMode() );
    RedakteurMode( rhs.RedakteurMode() );

    // Ja, auch das Rating sollte kopiert werden :)
    Rating( rhs.Rating() );
    
    if ( rhs.SchlagwoerterList != 0 ) {
      SchlagwoerterList = new QStrList( *rhs.SchlagwoerterList  );
      SchlagwoerterList->setAutoDelete( true );
    } else
      SchlagwoerterList = 0;

    if ( rhs.AusgabeList != 0 ) {
      AusgabeList = new QStrList( *rhs.AusgabeList  );
      AusgabeList->setAutoDelete( true );
    } else
      AusgabeList = 0;

    if ( rhs.AutorList != 0 ) {
      AutorList = new QStrList( *rhs.AutorList  );
      AutorList->setAutoDelete( true );
    } else
      AutorList = 0;

    if ( rhs.JahrgangList != 0 ) {
      JahrgangList = new QStrList( *rhs.JahrgangList  );
      JahrgangList->setAutoDelete( true );
    } else
      JahrgangList = 0;

    if ( rhs.RedakteurList != 0 ) {
      RedakteurList = new QStrList( *rhs.RedakteurList  );
      RedakteurList->setAutoDelete( true );
    } else
      RedakteurList = 0;

    
  } // selfdestruction
 
  // cout << "SearchMask CopyConstructor ende" << endl;
}

// operator=
SearchMask& SearchMask::operator=( SearchMask const & rhs )
{
  // cout << "SearchMask =operator" << endl;
  if ( & rhs != this )
    {
      Schlagwoerter( rhs.Schlagwoerter() );
      Jahrgang( rhs.Jahrgang() );
      Autor( rhs.Autor() );
      Redakteur( rhs.Redakteur() );
      Ausgabe( rhs.Ausgabe() );
      Zeitschrift( rhs.Zeitschrift() );
      
      setCaseSensitive( rhs.isCaseSensitive() );
      SchlagwoerterMode( rhs.SchlagwoerterMode() );
      AutorMode( rhs.AutorMode() );
      RedakteurMode( rhs.RedakteurMode() );


      if ( rhs.SchlagwoerterList != 0 ) {
	SchlagwoerterList = new QStrList( *rhs.SchlagwoerterList );
	SchlagwoerterList->setAutoDelete( true );
      } else
	SchlagwoerterList = 0;  
      
      if ( rhs.AusgabeList != 0 ) {
	AusgabeList = new QStrList( *rhs.AusgabeList  );
	AusgabeList->setAutoDelete( true );
      } else
	AusgabeList = 0;
      
      if ( rhs.AutorList != 0 ) {
	AutorList = new QStrList( *rhs.AutorList  );
	AutorList->setAutoDelete( true );
      } else
	AutorList = 0;
      
      if ( rhs.JahrgangList != 0 ) {
	JahrgangList = new QStrList( *rhs.JahrgangList  );
	JahrgangList->setAutoDelete( true );
      } else
	JahrgangList = 0;
      
      if ( rhs.RedakteurList != 0 ) {
	RedakteurList = new QStrList( *rhs.RedakteurList  );
	RedakteurList->setAutoDelete( true );
      } else
	RedakteurList = 0;
      
    } // selfdestruction
  
  return *this;
}

void SearchMask::Schlagwoerter( char const * a )   
{ 
  delete[] Schlagwoerter_; 
  Schlagwoerter_ = new char[strlen(a) + 1];
  strcpy( Schlagwoerter_, Entry::convert( a ) ); 
}

void SearchMask::Jahrgang( char const * a )
{ 
  delete[] Jahrgang_;
  Jahrgang_= new char[strlen(a) + 1];
  strcpy( Jahrgang_, Entry::convert( a ) );
}

void SearchMask::Autor( char const * a )    
{ 
  delete[] Autor_; 
  Autor_= new char[strlen(a) + 1]; 
  strcpy( Autor_, Entry::convert( a ) ); 
}

void SearchMask::Redakteur( char const * a )       
{ 
  delete[] Redakteur_; 
  Redakteur_= new char[strlen(a) + 1]; 
  strcpy( Redakteur_, Entry::convert( a ) ); 
}

void SearchMask::Ausgabe( char const * a )  
{ 
  delete[] Ausgabe_;
  Ausgabe_= new char[strlen(a) + 1];
  strcpy( Ausgabe_, Entry::convert( a ) ); 
}
 

void SearchMask::parse( void )
{
  // jeden Teil zerlegen

  //  
  //  Schlagwoerter
  //

  if ( !isEmpty( Schlagwoerter()) ) {
    // alte Liste freigeben
    if ( SchlagwoerterList != 0 ) {
      delete SchlagwoerterList;
    } 
    SchlagwoerterList = new QStrList( true ); // deep copies
    SchlagwoerterList->setAutoDelete( true );

    // einen findet man auf jeden Fall, auch wenn
    // keiner der Delimiter enthalten ist
    char * p = strtok( (char *) Schlagwoerter(), " ;,\t" );
    do {
      SchlagwoerterList->append( p ); // legt automatisch speicher an (deep copy)
    } while ( ( p = strtok( 0, " ;,\t" ) ) != 0 );
  }

  //  
  //  Autor
  //

  if ( !isEmpty( Autor()) ) {
    // alte Liste freigeben
    if ( AutorList != 0 ) {
      delete AutorList;
    } 
    AutorList = new QStrList( true ); // deep copies
    AutorList->setAutoDelete( true );

    // einen findet man auf jeden Fall, auch wenn
    // keiner der Delimiter enthalten ist
    char * p = strtok( (char *) Autor(), " ;,\t" );
    do {
      AutorList->append( p ); // legt automatisch speicher an (deep copy)
    } while ( ( p = strtok( 0, " ;,\t" ) ) != 0 );
  }

  //  
  //  Redakteur
  //

  if ( !isEmpty( Redakteur()) ) {
    // alte Liste freigeben
    if ( RedakteurList != 0 ) {
      delete RedakteurList;
    } 
    RedakteurList = new QStrList( true ); // deep copies
    RedakteurList->setAutoDelete( true );

    // einen findet man auf jeden Fall, auch wenn
    // keiner der Delimiter enthalten ist
    char * p = strtok( (char *) Redakteur(), " ;,\t" );
    do {
      RedakteurList->append( p ); // legt automatisch speicher an (deep copy)
    } while ( ( p = strtok( 0, " ;,\t" ) ) != 0 );
  }


  //  
  //  Jahrgang
  //

  if ( !isEmpty( Jahrgang()) ) {
    // alte Liste freigeben
    if ( JahrgangList != 0 ) {
      delete JahrgangList;
    } 
    JahrgangList = new QStrList( true ); // deep copies
    JahrgangList->setAutoDelete( true );

    // einen findet man auf jeden Fall, auch wenn
    // keiner der Delimiter enthalten ist
    char * p = strtok( (char *) Jahrgang(), " ;,\t" );
    do {
      JahrgangList->append( p ); // legt automatisch speicher an (deep copy)
    } while ( ( p = strtok( 0, " ;,\t" ) ) != 0 );
  }

  //  
  //  Ausgabe
  //

  if ( !isEmpty( Ausgabe()) ) {
    // alte Liste freigeben
    if ( AusgabeList != 0 ) {
      delete AusgabeList;
    } 
    AusgabeList = new QStrList( true ); // deep copies
    AusgabeList->setAutoDelete( true );

    // einen findet man auf jeden Fall, auch wenn
    // keiner der Delimiter enthalten ist
    char * p = strtok( (char *) Ausgabe(), " ;,\t" );
    do {
      AusgabeList->append( p ); // legt automatisch speicher an (deep copy)
    } while ( ( p = strtok( 0, " ;,\t" ) ) != 0 );
  }
}




