/* Emacs-Mode: -*-C++-*- */

/*
 *  Name         : searchmask.h
 *  Aufgabe      : Speicherung der Suchkriterien
 *  Version      : $Revision: 1.7 $
 *  Datum        : $Date: 1998/07/01 08:46:59 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef SEARCHMASK_H
#define SEARCHMASK_H

#include <qstrlist.h>

#include "entry.h"

/** Klasse zur Speicherung der Kriterien, nach
   denen die Register-Datei durchsucht werden soll */
class SearchMask
{
public:

  /** Konstruktor: Initialisierung ?? */
  // default: Kein CaseSensitve
  SearchMask( bool cs = false );
  ~SearchMask();
  SearchMask( SearchMask const & );
  SearchMask& SearchMask::operator=(SearchMask const & );
  
  
  /** Flag fuer Case-Sensitive Suche setzen */
  void setCaseSensitive( bool const cs ) { caseSensitive = cs; }

  /** Flag fuer Case-Sensitive Suche abfragen */
  bool isCaseSensitive( void ) const { return caseSensitive; };

  //
  // Element Modifier Funktionen
  //
  void Schlagwoerter( char const * );
  void Jahrgang( char const * );
  void Autor( char const * );
  void Redakteur( char const * );
  void Ausgabe( char const * );
  void Zeitschrift( Entry::MagazinTyp const a ) { Zeitschrift_ = a; }
  void Rating( int const r ) { Rating_ = r; }

  // Vorbemerkung fuer die Modi:
  // true -> AND Verknuepfung
  // false -> OR Verknuepfung

  void SchlagwoerterMode ( bool const m ) { SchlagwoerterMode_ = m; }
  void AutorMode( bool const m ) { AutorMode_ = m; }
  void RedakteurMode( bool const m ) { RedakteurMode_ = m; }
  
  //
  // Element Access Funktionen
  //
  char const * Autor() const { return Autor_; }
  char const * Redakteur() const { return Redakteur_; }
  char const * Ausgabe() const { return Ausgabe_; }
  Entry::MagazinTyp Zeitschrift() const { return Zeitschrift_; }
  char const * Jahrgang() const { return Jahrgang_; }
  char const * Schlagwoerter() const { return Schlagwoerter_; }
  int  const   Rating() const { return Rating_; }

  bool SchlagwoerterMode( void ) const { return SchlagwoerterMode_ ; }
  bool AutorMode( void ) const { return AutorMode_; }
  bool RedakteurMode( void ) const { return RedakteurMode_; }

  // Zerlegt alle belegten Felder in QList'en, damit anschliessend einfacher gesucht werden kann
  void parse();

  QStrList *SchlagwoerterList;
  QStrList *AutorList;
  QStrList *JahrgangList;
  QStrList *RedakteurList;
  QStrList *AusgabeList;
  
private:
  /// Flag fuer Case-Sensitive Suche
  bool caseSensitive;

  char  * Schlagwoerter_;
  char  * Autor_;
  char  * Redakteur_;
  char  * Ausgabe_;
  char  * Jahrgang_;
  Entry::MagazinTyp Zeitschrift_;

  bool SchlagwoerterMode_ ;
  bool AutorMode_;
  bool RedakteurMode_;

  /// Wert zwischen 0 - 100 , prozentual , Textmatch 
  /// 100 default, -1 -> kein Textsensitive Suche
  int   Rating_;   
};

#endif /* SEARCHMASK_H */
