// Mode: -*-C++-*-


/*
 *  Name         : toplevel.h
 *  Aufgabe      : Toplevel-Widget von KHeisePro
 *  Version      : $Revision: 1.19 $
 *  Datum        : $Date: 1999/02/08 22:01:28 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@fz-juelich.de>
 *                     Martin Habbecke   <M.Habbecke@fz-juelich.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef TOPLEVEL_H
#define TOPLEVEL_H


#include <qwidget.h>
#include <ktopwidget.h>

class KMenuBar;
class KToolBar;
class KStatusBar;
class QPopupMenu;
class MainWidget;
class SearchDialog;
class OptionsDialog;
class Database;
class Found;
class SearchMask;
class QString;
class QPrinter;
class QFontMetrics;
class KFM;
class QDate;
class KAccel;
class QFile;


/** Klasse des in KTopLevelWidget eingebetteten
  TopLevels */
class TopLevel : public KTopLevelWidget
{
  Q_OBJECT

public:
  /** Konstruktor: Anlegen des Widgets inkl. Menubar, Toolbar, Statusbar*/
  TopLevel( const char *name = 0 );

  /** Destruktor: */
  ~TopLevel();

  //#ifndef HAVE_KKEYCONF_H
  /** Funktion zur Rueckgabe des KAccel von TopLevel */
  //  inline KAccel *getAccel( void ) { return Accel; }
  //#endif /* HAVE_KKEYCONF_H */

public slots:
  /** Slot zum setzen einer Datei als neues Register */
  void newRegister( const char * );

  /* Slots der Menu-Items */
  /** Slot zum Oeffnen der Registerdatei */
  void openRegister();
  
  /** Setzen des Registernamens => Anlegen von Database */
  void setRegister( const QString & );
  
  /** Slot zum Schliessen der Registerdatei */
  void closeRegister();

  /** Slot zum updaten der Registerdatei via FTP */
  void updateRegister();

  /** Slot zum Anhaengen von Registerdaten */
  void appendData();

  /** Slot zum einstellen und konfigurieren des Druckers */
  void printerConfigure();

  /** Slot des Menuepunkts ASCII-Export->Gefundene Eintraege */
  void exportMenuAllItems();

  /** Slot des Menuepunkts ASCII-Export->Vorgemerkte Eintraege */
  void exportMenuPreItems();

  /** Slot zum Exportieren  aller Eintraege (Kontext-Menue) */
  void exportAllItemsASCII();
  
  /** Slot zum Exportieren aller markierten Eintraege (Kontext-Menue) */
  void exportMarkedItemsASCII();

  /** Slot des Menuepunkts Drucken->Gefundene Eintraege */
  void printMenuAllItems();

  /** Slot des Menuepunkts Drucken->Vorgemerkte Eintraege */
  void printMenuPreItems();

  /** Slot zum Drucken aller gefundenen Eintraege */
  void printAllItems();

  /** Slot zum Drucken des aktuellen Eintrages */
  void printMarkedItems();

  /** Slot zum Beenden des Programms */
  void exitProgram();

  /** Slot zum kopieren der markierten Eintraege in das
      X-Clipboard */
  void copyToClipboard();

  /** Slot zum Anzeigen des Such-Dialoges */
  void searchDlg();

  /** Slot zum Starten einer festgelegten Suche */
  void startSearch();

  /** Slot zum Aufrufen des Options-Dialoges */
  void showOptions();

  /** Slot zum konfigurieren des Keyboards */
  void configureKeyboard();
  
  /** Slot zum Anzeigen/Verstecken des Toolbars */
  void toggleToolbar();

  /** Slot zum Anzeigen/Verstecken des Statusbars */
  void toggleStatusbar();

  /** Slot der About-Box */
  void about();

  /** Slot der Hilfe */
  void help();

  /** Behandlung des headerClicked() Signals */
  void handleHeaderClicked( int );

  /** Behlandlung des "currentChanged" Signals von
      FoundList => setzen eines neuen Eintrags
      im ShowEntry */
  void setNewEntryInShowEntry( int );

  /** Unzipen der frisch geholten Registerdatei */
  void unzipRegister();

  /** Behandlung eines Klicks auf den TabBar */
  void setNewTab( int );

  /** Behandlung des KontextMenue-Punktes: Alle Eintraege vormerken */
  void addAllToPreselect();

  /** Behandlung des KontextMenue-Punktes: Markierte Eintraege vormerken */
  void addMarkedToPreselect();

  /** Behandlung des KontextMenue-Punktes: Alle Eintraege aus Vormerkung loeschen */
  void removeAllFromPreselect();

  /** Behandlung des KontextMenue-Punktes: Markierte Eintraege aus Vormerkung loeschen */
  void removeMarkedFromPreselect();

  /** Behandlung eines Klicks auf die CtRom-Checkbox im OptionsDialog */
  void receiveSupportCtrom( bool );

  /** Oeffnen des KFM mit dem gewuenschten Artikel */
  void openArticle();


protected:
  /** Funktion, die den Resize-Event behandelt */
  //  void resizeEvent( QResizeEvent *Event );



private:
  /** Funktion zur Erzeugung des Menubars */
  void setupMenu( void );

  /** Funktion zur Erzeugung des Toolbars */
  void setupToolbar( void );

  /** Funktion zur Erzeugung des Statusbars */
  void setupStatusbar( void );

  /** Erzeugen der Menu-Acceleratoren */
  void bindKeys( void ) ;

  /** Updaten der Accelerator-Strings in den Menu-Eintraegen */
  void updateMenuAccel( void );

#ifdef HAVE_KKEYCONF_H
  /** Hilfsfunktion zu updateMenuAccel */
  void changeMenuAccel ( QPopupMenu *, int ,  const char * );
#endif /* HAVE_KKEYCONF_H */

  /** Funktion zum Exportieren eines Found-Vektors als
      ASCII-Text 
      Return-Wert: 0 - Ok
                   1 - cancel
                   2 - error */
  int exportFoundASCII( Found * );

  /** Funktion zum Drucken eines kompletten Found-Vektors */
  void printFound( Found * );
  
  /** Funktion zur Ueberpruefung, ob die Registerdatei ein
      Update braucht */
  void checkUpdateTime( void );

  /** Funktion zum verschieben einer Datei ueber File-System-Grenzen
      hinaus */
  bool moveFile( const char *, const char * );

  /** Funktion zum einpassen eines Strings fuer einen gegebenen
      Font in eine gewuenschte Breite */
  void shrinkToFit( char *, int, QFontMetrics & );

  /** Das Mainwidget */
  MainWidget *mainWidget;

  /** Menuleiste */
  KMenuBar *MenuBar;

  /** Toolbar */
  KToolBar *ToolBar;

  /** Statuszeile */
  KStatusBar *StatusBar;

  /** Dialogfeld fuer die Festlegung einer Suche */
  SearchDialog *searchDialog;

  /** Dialogfeld der Voreinstellungen */
  OptionsDialog *optionsDialog;


  /** Menues */
  QPopupMenu *m_File, *m_Ascii, *m_Print, *m_Search, *m_Options, *m_Help;

  /** Variablen der MenuIDs */
  int openID, closeID, updateID, appendID, exportAllID, exportPreID, printAllID, printPreID;
  int printConfID, exitID, searchID, preAllID, preMarkedID, remAllID, remMarkedID;
  int optionsID, confkeysID, toolbarID, statusbarID, aboutID, helpID;
  

  /** Bool-Variablen fuer Toolbar, Statusbar */
  bool showToolbar, showStatusbar;

#ifndef HAVE_KKEYCONF_H
  /** Acceleratoren der Menues */
  KAccel *Accel;
#endif /* HAVE_KKEYCONF_H */

  /** aktuelle Datenbank , 0 -> es gibt keine */
  Database *database;
  
  /** Aktueller Found-Vektor */
  Found *actFound;
  /** Platz fuer die gefundenen Eintraege */
  /* Wird in der Database klasse gefuellt */
  Found *found;
  /** Liste der Vorgemerkten Eintraege */
  Found *markedFound;

  /** Suchparameter */
  /* wird in dem Such-Dialog benutzt */
  SearchMask *searchmask;

  /** Paint-Device fuer den Printer */
  /* Globales, damit einstellungen erhalten bleiben */
  QPrinter *printer;

  /** KFM-Instanz fuer den RegisterUpdate via FTP */
  KFM *kfm;

  /** Temporaere Datei fuer den Update */
  char *tmpfile;


  /** Datum des letzten RegisterUpdates */
  QDate *lastUpdate;

  /** Separator-String zur Trennung der Entry-Felder im
      X-Clipboard */
  char *clipSeparator;
};



#endif /* TOPLEVEL_H */





